/*
 * File:   flash.c
 * Author: michal
 *
 * Created on 1 października 2016, 18:28
 */

#include "xc.h"
#include "flash.h"
#include "ports.h"
#include "ui.h"
#include "libpic30.h"

extern volatile int g_key_numb; //TODO remove??

////////////////////////////////////////////////////////////////////////////
//    wave memory    ///////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

void __attribute__((__interrupt__, no_auto_psv)) _U1RXInterrupt(void)
{
    int32_t wave_num;
    int32_t wav_mem_address;
    int i_blocks4kB;
    int buffer256[256]; //buffer - decouples uart and spi timmings for 256B chunks
    int not_ready;  //check if memory is ready
    int i_mem;
    int j_mem;
    int i_timeout;
    int i_pream;
    int pream_ok;
    int asdf;
    extern int led_matrix[LED_SIZE_X * LED_SIZE_Y];

    led_matrix[LED_BUSY] = 1;
    //This is preamble that prevents executing rest of the isr if some junk occures on uart when usb is connected
    if(U1RXREG == START_BYTE)
        for(i_timeout = 0, i_pream = 0, pream_ok = 0; (i_timeout < 10000000) && !pream_ok; ++i_timeout)
        {
            if(U1STAbits.URXDA)
            {
                if(PREAM_BYTE != U1RXREG)
                {
                    IFS0bits.U1RXIF = 0;
                    return;
                }
                ++i_pream;
                if(i_pream == PREAM_LEN)
                    pream_ok = 1;
            }
        }
    else
    {
        IFS0bits.U1RXIF = 0;
        return;
    }
    
    
    
    while(!U1STAbits.URXDA);
    wave_num = U1RXREG - '0';
   
    ///////////////////write enable/////////////
    WAVMEM_CS_LAT = 1;
    asdf = SPI1BUF; //clear SPIRBF (receive buffer full) by reading SPI1BUF
    wait_transmit();
    WAVMEM_CS_LAT = 0;
    SPI1BUF = SPI_MEM_WRITE_EN; //send command to memory - write enable
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    WAVMEM_CS_LAT = 1;

    ///////////erase/////////////////
    for(i_blocks4kB = 0; i_blocks4kB < 4; ++i_blocks4kB)    //one waveform is 16kB (4 * 4kB block)
    {
        wav_mem_address = wave_num * 0x4000 + i_blocks4kB * 0x1000;
        mem_erase_4k(wav_mem_address);
    }

    for(j_mem = 0; j_mem < 64; ++j_mem)
    {
        ///////////////////write enable/////////////
        WAVMEM_CS_LAT = 1;
        asdf = SPI1BUF; //clear SPIRBF (receive buffer full) by reading SPI1BUF
        wait_transmit();
        WAVMEM_CS_LAT = 0;
        SPI1BUF = SPI_MEM_WRITE_EN; //send command to memory - write enable
        wait_transmit();
        wait_receive();
        asdf = SPI1BUF;
        WAVMEM_CS_LAT = 1;
        
        ////////////////send address////////////
        wav_mem_address = wave_num * 0x4000 + j_mem * 256;

        WAVMEM_CS_LAT = 1;
        asdf = SPI1BUF; //clear SPIRBF (receive buffer full) by reading SPI1BUF
        wait_transmit();
        WAVMEM_CS_LAT = 0;
        SPI1BUF = SPI_MEM_WRITE;   //send command WRITE
        wait_transmit();
        wait_receive();
        asdf = SPI1BUF;
        SPI1BUF = (wav_mem_address >> 16) & 0xFF;   //address b23-16
        wait_transmit();
        wait_receive();
        asdf = SPI1BUF;
        SPI1BUF = (wav_mem_address >> 8) & 0xFF;   //address b15-8
        wait_transmit();
        wait_receive();
        asdf = SPI1BUF;
        SPI1BUF = (wav_mem_address >> 0) & 0xFF;   //address b7-0
        wait_transmit();
        wait_receive();
        asdf = SPI1BUF;
        
        for(i_mem = 0; i_mem < 256; ++i_mem)    //take 256 bytes to buffer
        {
            while(!U1STAbits.URXDA);
            buffer256[i_mem] = U1RXREG;
        }
        for(i_mem = 0; i_mem < 256; ++i_mem)    //save 256 bytes from buffer to flash
        {
            SPI1BUF = buffer256[i_mem];
            wait_transmit();
            wait_receive();
            asdf = SPI1BUF;
        }
        WAVMEM_CS_LAT = 1;

        not_ready = 1;
        while(not_ready)
        {
            WAVMEM_CS_LAT = 1;
            asdf = SPI1BUF;
            wait_transmit();
            WAVMEM_CS_LAT = 0;
            SPI1BUF = SPI_MEM_STATUS1; //send command to memory - check status
            wait_transmit();
            wait_receive();
            asdf = SPI1BUF;
            SPI1BUF = 0x00; //transmit anything to clock data out of memory
            wait_transmit();
            wait_receive();
            not_ready = SPI1BUF & 0x01;
            WAVMEM_CS_LAT = 1;
        }
    }
    asdf = U1RXREG;
    led_matrix[LED_BUSY] = 0;
    IFS0bits.U1RXIF = 0;
}


void mem_read_16b(int *data_buf, int32_t wav_mem_address)
{
    int i_mem;
    int asdf;
    
    WAVMEM_CS_LAT = 1;
    asdf = SPI1BUF; //clear SPIRBF (receive buffer full) by reading SPI1BUF
    wait_transmit();
    WAVMEM_CS_LAT = 0;
    SPI1BUF = SPI_MEM_READ_ARRAY;   //send command READ
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    SPI1BUF = (wav_mem_address >> 16) & 0xFF;   //address b23-16
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    SPI1BUF = (wav_mem_address >> 8) & 0xFF;   //address b15-8
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    SPI1BUF = (wav_mem_address >> 0) & 0xFF;   //address b7-0
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    for(i_mem = 0; i_mem < (BLOCK_SIZE / 2); ++i_mem)   //read 4kB (two bytes per loop)
    {
        SPI1BUF = 0x00; //transmit anything to clock data out of memory
        wait_transmit();
        wait_receive();
        data_buf[i_mem] = SPI1BUF;
        SPI1BUF = 0x00;
        wait_transmit();
        wait_receive();
        data_buf[i_mem] |= SPI1BUF << 8;
    }
    WAVMEM_CS_LAT = 1;
}


void mem_erase_4k(int32_t block4k_mem_address)
{
    int not_ready;  //check if memory is ready
//    uint32_t i_wait;
    int asdf;

    //write enable
    WAVMEM_CS_LAT = 1;
    asdf = SPI1BUF; //clear SPIRBF (receive buffer full) by reading SPI1BUF
    wait_transmit();
    WAVMEM_CS_LAT = 0;
    SPI1BUF = SPI_MEM_WRITE_EN; //send command to memory - write enable
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    WAVMEM_CS_LAT = 1;
    
    //erase
    WAVMEM_CS_LAT = 1;
    asdf = SPI1BUF; //clear SPIRBF (receive buffer full) by reading SPI1BUF
    wait_transmit();
    WAVMEM_CS_LAT = 0;
    SPI1BUF = SPI_MEM_BLOCK_ERASE_4KB;   //send command ERASE
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    SPI1BUF = (block4k_mem_address >> 16) & 0xFF;   //address b23-16
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    SPI1BUF = (block4k_mem_address >> 8) & 0xFF;   //address b15-8
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    SPI1BUF = (block4k_mem_address >> 0) & 0xFF;   //address b7-0
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;     
    WAVMEM_CS_LAT = 1;

    not_ready = 1;
    while(not_ready)
    {
        WAVMEM_CS_LAT = 1;
        asdf = SPI1BUF;
        wait_transmit();
        WAVMEM_CS_LAT = 0;
        SPI1BUF = SPI_MEM_STATUS1; //send command to memory - check status
        wait_transmit();
        wait_receive();
        asdf = SPI1BUF;
        SPI1BUF = 0x00; //transmit anything to clock data out of memory
        wait_transmit();
        wait_receive();
        not_ready = SPI1BUF & 0x01;
        WAVMEM_CS_LAT = 1;
    }
}


void mem_write_page(int *page_buf, int32_t mem_address)
{
    int not_ready;  //check if memory is ready
    int i_mem;
    int asdf;

    ///////////////////write enable/////////////
    WAVMEM_CS_LAT = 1;
    asdf = SPI1BUF; //clear SPIRBF (receive buffer full) by reading SPI1BUF
    wait_transmit();
    WAVMEM_CS_LAT = 0;
    SPI1BUF = SPI_MEM_WRITE_EN; //send command to memory - write enable
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    WAVMEM_CS_LAT = 1;

    ////////////////send address////////////        
    WAVMEM_CS_LAT = 1;
    asdf = SPI1BUF; //clear SPIRBF (receive buffer full) by reading SPI1BUF
    wait_transmit();
    WAVMEM_CS_LAT = 0;
    SPI1BUF = SPI_MEM_WRITE; //send command WRITE
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    SPI1BUF = (mem_address >> 16) & 0xFF; //address b23-16
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    SPI1BUF = (mem_address >> 8) & 0xFF; //address b15-8
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    SPI1BUF = (mem_address >> 0) & 0xFF; //address b7-0
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;

    for (i_mem = 0; i_mem < 256; ++i_mem) //save 256 bytes from buffer to flash
    {
        SPI1BUF = page_buf[i_mem];
        wait_transmit();
        wait_receive();
        asdf = SPI1BUF;
    }
    WAVMEM_CS_LAT = 1;

    not_ready = 1;
    while (not_ready)
    {
        WAVMEM_CS_LAT = 1;
        asdf = SPI1BUF;
        wait_transmit();
        WAVMEM_CS_LAT = 0;
        SPI1BUF = SPI_MEM_STATUS1; //send command to memory - check status
        wait_transmit();
        wait_receive();
        asdf = SPI1BUF;
        SPI1BUF = 0x00; //transmit anything to clock data out of memory
        wait_transmit();
        wait_receive();
        not_ready = SPI1BUF & 0x01;
        WAVMEM_CS_LAT = 1;
    }
}

void mem_read_page(int *page_buf, int32_t mem_address)
{
    int i_mem;
    int asdf;
    
    WAVMEM_CS_LAT = 1;
    asdf = SPI1BUF; //clear SPIRBF (receive buffer full) by reading SPI1BUF
    wait_transmit();
    WAVMEM_CS_LAT = 0;
    SPI1BUF = SPI_MEM_READ_ARRAY;   //send command READ
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    SPI1BUF = (mem_address >> 16) & 0xFF;   //address b23-16
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    SPI1BUF = (mem_address >> 8) & 0xFF;   //address b15-8
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    SPI1BUF = (mem_address >> 0) & 0xFF;   //address b7-0
    wait_transmit();
    wait_receive();
    asdf = SPI1BUF;
    for(i_mem = 0; i_mem < 256; ++i_mem)   //read 256B
    {
        SPI1BUF = 0x00; //transmit anything to clock data out of memory
        wait_transmit();
        wait_receive();
        page_buf[i_mem] = SPI1BUF;
    }  
    WAVMEM_CS_LAT = 1;
}

void wait_transmit(void)
{
    while(SPI1STATbits.SPITBF); //wait until transmit buffer empty
}

void wait_receive(void)
{
    while(!SPI1STATbits.SPIRBF);    //wait for data in receive buffer
}


////////////////////////////////////////////////////////////////////////////
//    modulation memory    /////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

void modmem_write(int uart_mod_start_byte)
{
    int32_t wave_num;
    int32_t wav_mem_address;
    int i_blocks4kB;
    int buffer256[256]; //buffer - decouples uart and spi timmings for 256B chunks
    int i_mem;
    int j_mem;
    int i_timeout;
    int i_pream;
    int pream_ok;
    int asdf;
    
    //This is preamble that prevents executing rest of the function if some junk occures on uart when usb is connected
    if(uart_mod_start_byte == START_BYTE)
        for(i_timeout = 0, i_pream = 0, pream_ok = 0; (i_timeout < 10000000) && !pream_ok; ++i_timeout)
        {
            if(U1STAbits.URXDA)
            {
                if(PREAM_BYTE != U1RXREG)
                {
                    IFS0bits.U1RXIF = 0;
                    return;
                }
                ++i_pream;
                if(i_pream == PREAM_LEN)
                    pream_ok = 1;
            }
        }
    else
        return;
    
    
    
    
    while(!U1STAbits.URXDA);
    wave_num = U1RXREG - '0';
   
    ///////////////////write enable/////////////
    MODMEM_CS_LAT = 1;
    asdf = SPI2BUF; //clear SPIRBF (receive buffer full) by reading SPI2BUF
    modmem_wait_transmit();
    MODMEM_CS_LAT = 0;
    SPI2BUF = SPI_MEM_WRITE_EN; //send command to memory - write enable
    modmem_wait_transmit();
    modmem_wait_receive();
    asdf = SPI2BUF;
    MODMEM_CS_LAT = 1;
    
    ///////////erase/////////////////
    for(i_blocks4kB = 0; i_blocks4kB < 4; ++i_blocks4kB)    //one waveform is 16kB (4 * 4kB block)
    {
        wav_mem_address = wave_num * 0x4000 + i_blocks4kB * 0x1000;
        modmem_erase_4k(wav_mem_address);
    }

    for(j_mem = 0; j_mem < 64; ++j_mem)
    {
        ///////////////////write enable/////////////
        MODMEM_CS_LAT = 1;
        asdf = SPI2BUF; //clear SPIRBF (receive buffer full) by reading SPI2BUF
        modmem_wait_transmit();
        MODMEM_CS_LAT = 0;
        SPI2BUF = SPI_MEM_WRITE_EN; //send command to memory - write enable
        modmem_wait_transmit();
        modmem_wait_receive();
        asdf = SPI2BUF;
        MODMEM_CS_LAT = 1;
        
        ////////////////send address////////////
        wav_mem_address = wave_num * 0x4000 + j_mem * 256;

        MODMEM_CS_LAT = 1;
        asdf = SPI2BUF; //clear SPIRBF (receive buffer full) by reading SPI2BUF
        modmem_wait_transmit();
        MODMEM_CS_LAT = 0;
        SPI2BUF = SPI_MEM_WRITE;   //send command WRITE
        modmem_wait_transmit();
        modmem_wait_receive();
        asdf = SPI2BUF;
        SPI2BUF = (wav_mem_address >> 16) & 0xFF;   //address b23-16
        modmem_wait_transmit();
        modmem_wait_receive();
        asdf = SPI2BUF;
        SPI2BUF = (wav_mem_address >> 8) & 0xFF;   //address b15-8
        modmem_wait_transmit();
        modmem_wait_receive();
        asdf = SPI2BUF;
        SPI2BUF = (wav_mem_address >> 0) & 0xFF;   //address b7-0
        modmem_wait_transmit();
        modmem_wait_receive();
        asdf = SPI2BUF;
        
        for(i_mem = 0; i_mem < 256; ++i_mem)    //take 256 bytes to buffer
        {
            while(!U1STAbits.URXDA);
            buffer256[i_mem] = U1RXREG;
        }
        for(i_mem = 0; i_mem < 256; ++i_mem)    //save 256 bytes from buffer to flash
        {
            SPI2BUF = buffer256[i_mem];
            modmem_wait_transmit();
            modmem_wait_receive();
            asdf = SPI2BUF;
        }
        MODMEM_CS_LAT = 1;
        __delay_ms(3);    //wait for write
    }
    asdf = U1RXREG;
}

void modmem_erase_4k(int32_t block4k_mem_address)
{
    int asdf;

    //write enable
    MODMEM_CS_LAT = 1;
    asdf = SPI2BUF; //clear SPIRBF (receive buffer full) by reading SPI2BUF
    modmem_wait_transmit();
    MODMEM_CS_LAT = 0;
    SPI2BUF = SPI_MEM_WRITE_EN; //send command to memory - write enable
    modmem_wait_transmit();
    modmem_wait_receive();
    asdf = SPI2BUF;
    MODMEM_CS_LAT = 1;
    
    //erase
    MODMEM_CS_LAT = 1;
    asdf = SPI2BUF; //clear SPIRBF (receive buffer full) by reading SPI2BUF
    modmem_wait_transmit();
    MODMEM_CS_LAT = 0;
    SPI2BUF = SPI_MEM_BLOCK_ERASE_4KB;   //send command ERASE
    modmem_wait_transmit();
    modmem_wait_receive();
    asdf = SPI2BUF;
    SPI2BUF = (block4k_mem_address >> 16) & 0xFF;   //address b23-16
    modmem_wait_transmit();
    modmem_wait_receive();
    asdf = SPI2BUF;
    SPI2BUF = (block4k_mem_address >> 8) & 0xFF;   //address b15-8
    modmem_wait_transmit();
    modmem_wait_receive();
    asdf = SPI2BUF;
    SPI2BUF = (block4k_mem_address >> 0) & 0xFF;   //address b7-0
    modmem_wait_transmit();
    modmem_wait_receive();
    asdf = SPI2BUF;
    MODMEM_CS_LAT = 1;
    __delay_ms(350);    //wait for erase    
}

void modmem_wait_transmit(void)
{
    while(SPI2STATbits.SPITBF); //wait until transmit buffer empty
}

void modmem_wait_receive(void)
{
    while(!SPI2STATbits.SPIRBF);    //wait for data in receive buffer
}
