/*
 * File:   fpga_interface.c
 * Author: michal
 *
 * Created on 21 września 2016, 23:09
 */


#include "xc.h"
#include "ports.h"
#include "fpga_interface.h"
#include "flash.h"
#include <math.h>
#include <stdint.h>


extern struct wave_param_struct wave_param[2];


void send_word_fpga(int64_t data, int data_length, int addr)
{
    int i_data = 0;
    INT_WR_LAT = 0;
    for(i_data = 0; i_data < data_length; ++i_data)
    {
        INT_DCLK_LAT = 0;
        LATE &= ~(BUS_MASK);
        LATE |= BUS_MASK & data >> (BUS_SIZE * (data_length - i_data - 1));
        INT_DCLK_LAT = 1;
    }
    INT_DCLK_LAT = 0;
    LATE &= ~(BUS_MASK);
    LATE |= BUS_MASK & addr;
    INT_DCLK_LAT = 1;
    INT_DCLK_LAT = 0;
    INT_WR_LAT = 1;
    INT_WR_LAT = 0;
}

void refresh_wave(int channel, struct fpga_params_struct *fpga_params)
{
    double phase_acc_freq;   // sampling frequency to set in phase accumullator (before bumping by PLLs)
    double freq_sampl;   // required sampling frequency (determined by required wave frequency and memory step)
    int step;   //memory step (higher for higher wave frequencies)
    int64_t mod_clk_phase_acc_conf;
    int i_modpa;
    
    // variable frequency DDS mode
    if(fpga_params->dds_mode == 0)
    {
        freq_sampl = wave_param[channel].freqency * WAVE_MEM_LENGTH;   // sampling frequency required for given wave frequency
        
        //configure fpga - configure for a given frequency - compute minimum wave memory step
        for(step = 1; freq_sampl > FREQ_PLL_OUT_MAX; step *= 2)
            freq_sampl = freq_sampl / 2.0;  // decrease sampling frequency to possible value (step size compensates it)
        fpga_params->cnt_step = step;    // it's the required memory step
        //configure fpga - configure for a given frequency - compute phase accumullator increment and select PLLs
        if(freq_sampl < FREQ_PLL_OUT_MIN)
        {
            fpga_params->clk_pll_conf = PLL_OFF;
            phase_acc_freq = freq_sampl;
        }
        else
        {
            fpga_params->clk_pll_conf = PLL_EXT2;
            phase_acc_freq = freq_sampl / PLL_EXT_MULT;
        }
        
        fpga_params->clk_phase_acc_conf = (int64_t)(phase_acc_freq * (pow(2, FPGA_PHASE_ACC_BITS) / FPGA_PHASE_ACC_IN_CLK));    //compute phase accumulator increment for the frequency
    }
    // constant frequency DDS mode
    else
    {
        fpga_params->clk_phase_acc_conf = (int64_t)(wave_param[channel].freqency * (pow(2, FPGA_PHASE_ACC_BITS) / FPGA_PHASE_ACC_IN_CLK));    //compute phase accumulator increment for the frequency
        //in this dds mode disable pll
        fpga_params->clk_pll_conf = PLL_OFF;
    }
    
    //configure fpga - compute phase
    fpga_params->addr_phase = (int32_t)(wave_param[channel].phase * WAVE_MEM_LENGTH / 360.0);
    //configure fpga - compute amplitude
    fpga_params->amplitude = (int32_t) ((wave_param[channel].amplitude * analog_cal[channel].out_amplitude) * AMPLITUDE_RESOLUTION / OUT_VOLTAGE_MAX);
    //configure fpga - compute offset
    fpga_params->offset = (int32_t) ((0 - wave_param[channel].offset + analog_cal[channel].out_offset) * OFFSET_RESOLUTION / OUT_VOLTAGE_MAX  * analog_cal[channel].out_amplitude);

    //set modulation
    switch (wave_param[channel].mod_type)
    {
        case MOD_NONE:
            fpga_params->mod_am = 0x0;
            fpga_params->mod_fm = 0x0;
            fpga_params->mod_phase = 0x0;
            fpga_params->mod_pwm = 0x0;
            fpga_params->mod_ampl = (int32_t) (wave_param[channel].mod_ampl * 32767 / 100); //modulating wave in percent
            fpga_params->mod_dev_power = 0;
            break;
        case MOD_AM:
            fpga_params->mod_am = 0x1;
            fpga_params->mod_fm = 0x0;
            fpga_params->mod_phase = 0x0;
            fpga_params->mod_pwm = 0x0;
            fpga_params->mod_ampl = (int32_t) (wave_param[channel].mod_ampl * 32767 / 100); //modulating wave in percent
            fpga_params->mod_dev_power = 0;
            break;
        case MOD_AM_SC:
            fpga_params->mod_am = 0x2;
            fpga_params->mod_fm = 0x0;
            fpga_params->mod_phase = 0x0;
            fpga_params->mod_pwm = 0x0;
            fpga_params->mod_ampl = (int32_t) (wave_param[channel].mod_ampl * 32767 / 100); //modulating wave in percent
            fpga_params->mod_dev_power = 0;
            break;
        case MOD_FM:
            fpga_params->mod_am = 0x0;
            fpga_params->mod_fm = 0x1;
            fpga_params->mod_phase = 0x0;
            fpga_params->mod_pwm = 0x0;
            if(fpga_params->dds_mode == 0)
            {
                mod_clk_phase_acc_conf = WAVE_MEM_LENGTH / fpga_params->cnt_step * (int64_t)((wave_param[channel].freqency + wave_param[channel].mod_dev) * (pow(2, FPGA_PHASE_ACC_BITS) / FPGA_PHASE_ACC_IN_CLK)) - fpga_params->clk_phase_acc_conf;
                for(i_modpa = 0; mod_clk_phase_acc_conf > (pow(2, i_modpa) * pow(2, 15)); ++i_modpa)
                    ;
                fpga_params->mod_ampl = mod_clk_phase_acc_conf / pow(2, i_modpa);
                fpga_params->mod_dev_power = i_modpa;
            }
            else
            {
                mod_clk_phase_acc_conf = (int64_t)((wave_param[channel].freqency + wave_param[channel].mod_dev) * (pow(2, FPGA_PHASE_ACC_BITS) / FPGA_PHASE_ACC_IN_CLK)) - fpga_params->clk_phase_acc_conf;
                for(i_modpa = 0; mod_clk_phase_acc_conf > (pow(2, i_modpa) * pow(2, 15)); ++i_modpa)
                    ;
                fpga_params->mod_ampl = mod_clk_phase_acc_conf / pow(2, i_modpa);
                fpga_params->mod_dev_power = i_modpa;
            }
            break;
        case MOD_PM:
            fpga_params->mod_am = 0x0;
            fpga_params->mod_fm = 0x0;
            fpga_params->mod_phase = 0x1;
            fpga_params->mod_pwm = 0x0;
            fpga_params->mod_ampl = (int32_t) (wave_param[channel].mod_ampl * 32767 / 100); //modulating wave in percent
            fpga_params->mod_dev_power = 0;
            break;
        case MOD_PWM:
            fpga_params->mod_am = 0x0;
            fpga_params->mod_fm = 0x0;
            fpga_params->mod_phase = 0x0;
            fpga_params->mod_pwm = 0x1;
            fpga_params->mod_ampl = (int32_t) (wave_param[channel].mod_ampl * 32767 / 100); //modulating wave in percent
            fpga_params->mod_dev_power = 0;
            break;
    }
    fpga_params->mod_freq = (int64_t) (wave_param[channel].mod_freq * (pow(2, FPGA_PHASE_ACC_BITS) / FPGA_MOD_PHASE_ACC_IN_CLK));    //compute phase accumulator increment for the modulation frequency
    fpga_params->mod_wave_num = wave_param[channel].mod_wave_num;
    fpga_params->in_offset_data = 32767 * analog_cal[channel].in_offset;
    fpga_params->in_amplitude_data = 16384 * analog_cal[channel].in_amplitude;
    
    //select source of modulating signal
    if(wave_param[channel].mod_ext_not_int)
        fpga_params->mod_conf |= BIT_MOD_EXT;
    else
        fpga_params->mod_conf &= ~BIT_MOD_EXT;
    //select write or read modulating wave memory
    if(wave_param[channel].mod_write_not_read == 1)
        fpga_params->mod_conf |= BIT_MOD_MEM_RW;
    else
        fpga_params->mod_conf &= ~BIT_MOD_MEM_RW;
}

void refresh_fpga_memory(int wave_num, int channel)
{
    int i_mem;
    int32_t i_blocks4kB;
    int data_buf[2048];
    int32_t wav_mem_address;
    int32_t wav_num_32 = wave_num;  //need more bits
    
    for(i_blocks4kB = 0; i_blocks4kB < 4; ++i_blocks4kB)
    {
        wav_mem_address = wav_num_32 * 0x4000 + (i_blocks4kB * 0x1000); //3 bytes address
        mem_read_16b(data_buf, wav_mem_address);
        for(i_mem = 0; i_mem < (BLOCK_SIZE / 2); ++i_mem)   //read 4kB (two bytes per loop)
            send_word_fpga((0x0000FFFF & data_buf[i_mem]) | (0xFFFF0000 & ((i_blocks4kB * BLOCK_SIZE / 2 + i_mem) << 16)), 4, (channel == 0) ? FPGA_ADDR_WAVE_MEM_CH1 : FPGA_ADDR_WAVE_MEM_CH2);
    }
}

//DON'T USE THIS FUNCTION - CRASHES - TO BE FIXED OR REMOVED
void refresh_fast_sin_memory(int wave_num, int channel) //it's not faster :(
{
    int32_t i_mem;
    int i_buf;
    int data_buf[2048];
    int32_t wav_mem_address;
    int32_t wav_num_32 = wave_num;  //need more bits
    
    wav_mem_address = wav_num_32 * 0x4000 + 0x1000; //3 bytes address
    mem_read_16b(data_buf, wav_mem_address);    //read 1/4 of sine wave
    
    for(i_mem = 0; i_buf < (BLOCK_SIZE / 2); ++i_mem, ++i_buf)   //read 4kB (two bytes per loop)
        send_word_fpga((0x0000FFFF & data_buf[i_buf]) | (0xFFFF0000 & (i_mem << 16)), 4, (channel == 0) ? FPGA_ADDR_WAVE_MEM_CH1 : FPGA_ADDR_WAVE_MEM_CH2);
    for(;i_buf > 0; ++i_mem, --i_buf)   //read 4kB (two bytes per loop)
        send_word_fpga((0x0000FFFF & -data_buf[i_buf]) | (0xFFFF0000 & (i_mem << 16)), 4, (channel == 0) ? FPGA_ADDR_WAVE_MEM_CH1 : FPGA_ADDR_WAVE_MEM_CH2);
    for(; i_buf < (BLOCK_SIZE / 2); ++i_mem, ++i_buf)   //read 4kB (two bytes per loop)
        send_word_fpga((0x0000FFFF & -data_buf[i_buf]) | (0xFFFF0000 & (i_mem << 16)), 4, (channel == 0) ? FPGA_ADDR_WAVE_MEM_CH1 : FPGA_ADDR_WAVE_MEM_CH2);
    for(; i_buf > 0; ++i_mem, --i_buf)   //read 4kB (two bytes per loop)
        send_word_fpga((0x0000FFFF & data_buf[i_buf]) | (0xFFFF0000 & (i_mem << 16)), 4, (channel == 0) ? FPGA_ADDR_WAVE_MEM_CH1 : FPGA_ADDR_WAVE_MEM_CH2);
}

void generate_rect_memory(struct wave_param_struct wave_param, int channel)
{
    int32_t i_rect_gen;
    int16_t fixp_sym;   //symmetry as fixed point value
    
    fixp_sym = (int32_t)(wave_param.symmetry * WAVE_MEM_LENGTH);
    for (i_rect_gen = 0; i_rect_gen < WAVE_MEM_LENGTH; ++i_rect_gen)
        if(i_rect_gen < fixp_sym)
            send_word_fpga((0x0000FFFF & 0x7FFF) | (0xFFFF0000 & (i_rect_gen << 16)), 4, (channel == 0) ? FPGA_ADDR_WAVE_MEM_CH1 : FPGA_ADDR_WAVE_MEM_CH2);
        else
            send_word_fpga((0x0000FFFF & 0x8000) | (0xFFFF0000 & (i_rect_gen << 16)), 4, (channel == 0) ? FPGA_ADDR_WAVE_MEM_CH1 : FPGA_ADDR_WAVE_MEM_CH2);
}

void generate_tri_memory(struct wave_param_struct wave_param, int channel)
{
    int32_t i_tri_gen;
    int32_t wav_pt = 0;
    int32_t fixp_sym_divp;
    int chan_addr;
    double a1_coe;
    double a2_coe;
    const int b1_coe = -32768;
    const int b2_coe = 32767;
   
    chan_addr = (channel == 0) ? FPGA_ADDR_WAVE_MEM_CH1 : FPGA_ADDR_WAVE_MEM_CH2;
    fixp_sym_divp = (int32_t)(wave_param.symmetry * WAVE_MEM_LENGTH);
    a1_coe = 65535.0 / (wave_param.symmetry * WAVE_MEM_LENGTH);
    a2_coe = -65535.0 / ((1 - wave_param.symmetry) * WAVE_MEM_LENGTH);
    
    for (i_tri_gen = 0; i_tri_gen < fixp_sym_divp; ++i_tri_gen)
        if(32767 >= (wav_pt = ((int32_t)(a1_coe * i_tri_gen)) + b1_coe))    //float multiplied by 0.0 should be exactly 0.0
            send_word_fpga((0x0000FFFF & wav_pt) | (0xFFFF0000 & (i_tri_gen << 16)), 4, chan_addr);
        else
            send_word_fpga((0x0000FFFF & 32767) | (0xFFFF0000 & (i_tri_gen << 16)), 4, chan_addr);
    for (; i_tri_gen < WAVE_MEM_LENGTH; ++i_tri_gen)
        if(-32768 <= (wav_pt = ((int32_t)(a2_coe * (i_tri_gen - fixp_sym_divp))) + b2_coe))    //float multiplied by 0.0 should be exactly 0.0
            send_word_fpga((0x0000FFFF & wav_pt) | (0xFFFF0000 & (i_tri_gen << 16)), 4, chan_addr);
        else
            send_word_fpga((0x0000FFFF & -32768) | (0xFFFF0000 & (i_tri_gen << 16)), 4, chan_addr);
}                                        

void send_configuration_fpga(struct fpga_params_struct fpga_params, int channel)
{
    int64_t data_to_fpga;
    
    data_to_fpga = (0x00000000FFFFFFFF & fpga_params.clk_phase_acc_conf) | (0x0000000F00000000 & (fpga_params.clk_pll_conf << 32)) | (0x000000F000000000 & ((int64_t)fpga_params.mod_fm << 36));
    send_word_fpga(data_to_fpga, 5, (channel == 0) ? FPGA_ADDR_CLK_GEN_CH1 : FPGA_ADDR_CLK_GEN_CH2);
    
    data_to_fpga = 0x00000000FFFF0000 & (fpga_params.cnt_step << 16);
    send_word_fpga(data_to_fpga, 4, (channel == 0) ? FPGA_ADDR_WAVE_CNT_CH1 : FPGA_ADDR_WAVE_CNT_CH2);
    
    data_to_fpga = 0x00000000000000FF & fpga_params.mod_pwm;
    send_word_fpga(data_to_fpga, 1, (channel == 0) ? FPGA_ADDR_PWM_COMP_CH1 : FPGA_ADDR_PWM_COMP_CH2);
    
    data_to_fpga = (0x000000000000FFFF & fpga_params.offset) | (0x00000000FFFF0000 & (fpga_params.amplitude << 16)) | (0x0000000300000000 & ((int64_t)fpga_params.mod_am << 32));
    send_word_fpga(data_to_fpga, 5, (channel == 0) ? FPGA_ADDR_AMPL_OFFS_CH1 : FPGA_ADDR_AMPL_OFFS_CH2);
    
    data_to_fpga = (0x0F & fpga_params.dds_mode) | (0xF0 & (fpga_params.mod_phase << 4)) | (0x0000000000FFFF00 & (fpga_params.addr_phase << 8));
    send_word_fpga(data_to_fpga, 3, (channel == 0) ? FPGA_ADDR_DDS_MODE_CH1 : FPGA_ADDR_DDS_MODE_CH2);
    
    data_to_fpga = (0x000000000000FFFF & fpga_params.mod_ampl) | (0x00000000000F0000 & (fpga_params.mod_conf << 16)) | (0x0000000000F00000 & (fpga_params.mod_dev_power << 20)) | (0x00000000FF000000 & (fpga_params.mod_wave_num << 24));
    send_word_fpga(data_to_fpga, 4, (channel == 0) ? FPGA_ADDR_MOD_CONF_CH1 : FPGA_ADDR_MOD_CONF_CH2);

    data_to_fpga = (0x00000000FFFFFFFF & fpga_params.mod_freq);
    send_word_fpga(data_to_fpga, 4, (channel == 0) ? FPGA_ADDR_MOD_CONF_FREQ_CH1 : FPGA_ADDR_MOD_CONF_FREQ_CH2);

    data_to_fpga = (0x000000000000FFFF & fpga_params.in_offset_data) | (0x00000000FFFF0000 & (fpga_params.in_amplitude_data << 16));
    send_word_fpga(data_to_fpga, 4, (channel == 0) ? FPGA_ADDR_ADC_CAL_CH1 : FPGA_ADDR_ADC_CAL_CH2);

    data_to_fpga = (0x000000000000000F & fpga_params.trig_sel);
    send_word_fpga(data_to_fpga, 1, FPGA_ADDR_DIG_IO_MUX);
}

void refresh_configuration_all(struct fpga_params_struct *fpga_params)
{
    int channel;

    channel = 0; //update both channels
    refresh_wave(channel, fpga_params);
    send_configuration_fpga(*fpga_params, channel);
    
    channel = 1; //update both channels
    refresh_wave(channel, fpga_params);
    send_configuration_fpga(*fpga_params, channel);
}
