#include <stdint.h>
#include "structs.h"

#define PLL_OFF 0
#define PLL_EXT2 2

#define PLL_EXT_MULT 8

#define FREQ_PLL_OUT_MIN 16000000.0
#define FREQ_PLL_OUT_MAX 150000000.0

#define FPGA_ADDR_CLK_GEN_CH1 0
#define FPGA_ADDR_WAVE_CNT_CH1 1
#define FPGA_ADDR_WAVE_MEM_CH1 2
#define FPGA_ADDR_PWM_COMP_CH1 3
#define FPGA_ADDR_AMPL_OFFS_CH1 4
#define FPGA_ADDR_DDS_MODE_CH1 5
#define FPGA_ADDR_MOD_CONF_CH1 6
#define FPGA_ADDR_MOD_CONF_FREQ_CH1 7
#define FPGA_ADDR_ADC_CAL_CH1 8

#define FPGA_ADDR_CLK_GEN_CH2 20
#define FPGA_ADDR_WAVE_CNT_CH2 21
#define FPGA_ADDR_WAVE_MEM_CH2 22
#define FPGA_ADDR_PWM_COMP_CH2 23
#define FPGA_ADDR_AMPL_OFFS_CH2 24
#define FPGA_ADDR_DDS_MODE_CH2 25
#define FPGA_ADDR_MOD_CONF_CH2 26
#define FPGA_ADDR_MOD_CONF_FREQ_CH2 27
#define FPGA_ADDR_ADC_CAL_CH2 28

#define FPGA_ADDR_DIG_IO_MUX 40


#define FPGA_PHASE_ACC_BITS 33
#define FPGA_PHASE_ACC_IN_CLK 150000000.0
#define FPGA_MOD_PHASE_ACC_IN_CLK 20000000.0

#define WAVE_MEM_LENGTH 8192
#define AMPLITUDE_RESOLUTION (65536-1)  //should be 65536 but won't fit 16b, 65536-1 still may overflow due to float rounding but finally it doesn't matter as calibration will scale the amplitude word down (so it definitely will fit the 16b)
#define OFFSET_RESOLUTION (32768-1)     //1/2 of the above - because it's with sign

#define OUT_VOLTAGE_MAX 10

#define ADC_AMPLITUDE_RESOLUTION 4096

#define BUS_SIZE 8 //in bits
#define BUS_MASK 0x00FF    //must match DATA_BUS_SIZE

#define BIT_MOD_MEM_RW (1 << 0)
#define BIT_MOD_EXT (1 << 1)



extern struct analog_cal_struct analog_cal[2];


void send_word_fpga(int64_t data, int data_length, int addr);
void send_configuration_fpga(struct fpga_params_struct fpga_params, int channel);

void refresh_wave(int channel, struct fpga_params_struct *fpga_params);
void refresh_fpga_memory(int wave_num, int channel);
void refresh_fast_sin_memory(int wave_num, int channel);
void generate_rect_memory(struct wave_param_struct wave_param, int channel);
void generate_tri_memory(struct wave_param_struct wave_param, int channel);
void refresh_configuration_all(struct fpga_params_struct *fpga_params);
