/*
 * File:   init_uc.c
 * Author: michal
 *
 * Created on 18 września 2016, 20:09
 */

#include "init.h"
#include "xc.h"
#include "ports.h"
#include "lcd.h"
#include "flash.h"

void init_clock(void)
{
    OSCCON = 0x1100;
    CLKDIV = 0x0000;
    OSCTUN = 0x0000;
}

void init_ports(void)
{
    AD1PCFG = 0xffff; //digital ports on analog ports (the f***ing default was to make digital inputs unusable)
    
    LCD_DB4_TRIS = 0;
    LCD_DB5_TRIS = 0;
    LCD_DB6_TRIS = 0;
    LCD_DB7_TRIS = 0;
    LCD_E_TRIS = 0;
    LCD_RS_TRIS = 0;

    DIG_IN_50R_EN_TRIS = 0;
    DIG_OUT_50R_DIS_TRIS = 0;
    DIG_OUT_LO_Z_DIS_TRIS = 0;
    
    INT_DATA_0_TRIS = 0;
    INT_DATA_1_TRIS = 0;
    INT_DATA_2_TRIS = 0;
    INT_DATA_3_TRIS = 0;
    INT_DATA_4_TRIS = 0;
    INT_DATA_5_TRIS = 0;
    INT_DATA_6_TRIS = 0;
    INT_DATA_7_TRIS = 0;
    INT_WR_TRIS = 0;
    INT_DCLK_TRIS = 0;
    
    OUT_EN_A_TRIS = 0;
    OUT_EN_A_LAT = 0;
    OUT_ATTEN_DIS_A_TRIS = 0;
    OUT_ATTEN_DIS_A_LAT = 1;
    OUT_EN_B_TRIS = 0;
    OUT_EN_B_LAT = 0;
    OUT_ATTEN_DIS_B_TRIS = 0;
    OUT_ATTEN_DIS_B_LAT = 1;
    
    LED_X0_TRIS = 0;
    LED_X1_TRIS = 0;
    LED_X2_TRIS = 0;
    LED_X3_TRIS = 0;
    LED_Y0_TRIS = 0;
    LED_Y1_TRIS = 0;
    LED_Y2_TRIS = 0;
    LED_Y3_TRIS = 0;
    
    KEY_X0_TRIS = 0;
    KEY_X1_TRIS = 0;
    KEY_X2_TRIS = 0;
    KEY_X3_TRIS = 0;
    KEY_X4_TRIS = 0;
    KEY_X5_TRIS = 0;
    KEY_Y0_TRIS = 1;
    KEY_Y1_TRIS = 1;
    KEY_Y2_TRIS = 1;
    KEY_Y3_TRIS = 1;
    KEY_Y4_TRIS = 1;
    KEY_Y5_TRIS = 1;
    
    ROT_A_TRIS = 1;
    ROT_B_TRIS = 1;

    WAVMEM_CS_TRIS = 0;    
//    WAVMEM_SCLK_TRIS = 0;
//    WAVMEM_SO_TRIS = 0;
//    WAVMEM_SI_TRIS = 1;
    MODMEM_CS_TRIS = 0;
    
    TRISC = 0x00;
    AUX1_TRIS = 1;
    
    TRISAbits.TRISA0 = 0;
    TRISAbits.TRISA1 = 0;
    TRISAbits.TRISA2 = 0;
    TRISAbits.TRISA3 = 0;
    TRISAbits.TRISA4 = 0;
    TRISAbits.TRISA5 = 0;
    TRISAbits.TRISA9 = 0;
    TRISAbits.TRISA10 = 0;
    TRISBbits.TRISB3 = 0;
    TRISDbits.TRISD7 = 0;
    TRISDbits.TRISD13 = 0;
    TRISEbits.TRISE8 = 0;
    TRISEbits.TRISE9 = 0;
}

void init_timer(void)   //timer 1 is used for sweeping keyboard and multiplexing LEDs (generates interrupts)
{
    TMR1 = 0x00;
    PR1 = 0x00FF;
    IPC0bits.T1IP = 3;
    IFS0bits.T1IF = 0;
    IEC0bits.T1IE = 1;
    T1CONbits.TCKPS = 0;
    T1CONbits.TCS = 0;
    T1CONbits.TGATE = 0;
    T1CONbits.TON = 1;
}

void init_contrast_pwm(void)    //output compare feature is used as PWM for setting LCD contrast
{
    PR2 = 10000;    //pwm period = PR2*Tcy*timer_prescaler
    OC5RS = 1500;  //initial value of duty (fraction of PR5) - to be overwritten while calibration
    //OC5R = 10000;   //initial duty cycle?????????
    OC5CONbits.OCTSEL = 0;   //timer 2
    OC5CONbits.OCM = 0x6;   //PWM mode on OCx, Fault pin is disabled
    T2CONbits.TON = 1;  //timer 2 on
    T5CONbits.TON = 1;  //timer 2 on???????????
    
}

void init_spi_wave_memory(void) //configuration of SPI interface to work with external flash memory
{
    SPI1CON1bits.CKP = 1;
    SPI1CON1bits.CKE = 0;
    SPI1CON1bits.PPRE = 3;
    SPI1CON1bits.SPRE = 7;
    SPI1CON1bits.MSTEN = 1;
    SPI1STATbits.SPIROV = 0;
    SPI1STATbits.SPIEN = 1;
}

void init_spi_mod_memory(void) //configuration of SPI interface to work with external flash memory
{
    SPI2CON1bits.CKP = 1;
    SPI2CON1bits.CKE = 0;
    SPI2CON1bits.PPRE = 3;
    SPI2CON1bits.SPRE = 7;
    SPI2CON1bits.MSTEN = 1;
    SPI2STATbits.SPIROV = 0;
    SPI2STATbits.SPIEN = 1;
}

void init_uart(void)    //configuration of UART to enter ISR and receive waveform from computer when it sends the data
{
    U1MODEbits.UEN = 0;//2;
    U1MODEbits.BRGH = 0;
    U1MODEbits.PDSEL = 0;   // 8 bit, no parity
    U1MODEbits.STSEL = 0;   // one stop
//    U1STAbits.UTXEN = 1;
    U1STAbits.URXISEL = 0;  // interrupt on receive
    U1BRG = 103;            // had to be 19200 but works with 9600 - TODO recalculate
    IPC2bits.U1RXIP = 6;    //high priority
    IEC0bits.U1RXIE = 1;    // enable interrupt
    U1MODEbits.UARTEN = 1;  // enable uart
}

void init_lcd(void) //run up the LCD (HD44780)
{
    lcd_reset();
    lcd_send_instr(0x28);   // 4-bit operation, 1-line display, 5x8 font
    lcd_send_instr(0x0C);   // display on, no cursor
    lcd_send_instr(0x01);   // clear display
        volatile int i_wait;
        for(i_wait = 0; i_wait < 10000; ++i_wait);
    lcd_send_instr(0x06);   // shift cursor right
}

void init_calibration(struct analog_cal_struct *analog_cal) //take calibration data from external flash memory to variables
{
    int32_t mem_address;
    int page_buf[256];
    int i_data;

    int32_t cal_buf[8];
    int32_t int_tmp1, int_tmp2, int_tmp3, int_tmp4;
    
    mem_address = 0x07F000; //address of calibration data in external flash memory
    mem_read_page(page_buf, mem_address);   //read page (256 bits)
    
    for(i_data = 0; i_data < 8; ++i_data)   //take 8 words (32 bytes) of calibration data, convert and put into floating point buffer
    {
        int_tmp1 = 0xFF & page_buf[i_data * 4 + 3];
        int_tmp2 = 0xFF & page_buf[i_data * 4 + 2];
        int_tmp3 = 0xFF & page_buf[i_data * 4 + 1];
        int_tmp4 = 0xFF & page_buf[i_data * 4 + 0];
        cal_buf[i_data] =  (int_tmp1 << 24) | (int_tmp2 << 16) | (int_tmp3 << 8) | (int_tmp4 << 0);
    }
    OC5RS = (page_buf[i_data * 4] << 8);    //...and one byte to set LCD contrast
    float asdf=999;
    analog_cal[0].in_amplitude = *((double *)&cal_buf[0]);
    analog_cal[0].in_offset = *((double *)&cal_buf[1]);
    analog_cal[0].out_amplitude = (asdf= *((double *)&cal_buf[2]));
    analog_cal[0].out_offset = *((double *)&cal_buf[3]);
    analog_cal[1].in_amplitude = *((double *)&cal_buf[4]);
    analog_cal[1].in_offset = *((double *)&cal_buf[5]);
    analog_cal[1].out_amplitude = *((double *)&cal_buf[6]);
    analog_cal[1].out_offset = *((double *)&cal_buf[7]);
}
