/*
 * File:   lcd.c
 * Author: michal
 *
 * Created on 8 października 2016, 10:28
 */

#include "xc.h"
#include "lcd.h"
#include "ports.h"
#include <stdio.h>
#include <stdint.h>
#include "libpic30.h"

extern struct wave_param_struct wave_param[2];

void lcd_send_string(char *char_arr, int length, int line, int start_pos)
{
    int i_str;
    for(i_str = 0; i_str < length; ++i_str)
        lcd_send_char(char_arr[i_str], line, start_pos + i_str);
}

void lcd_send_char(char character, int line, int pos)
{
    int line_offset = 0;
    
    if(line == 1)
        line_offset = 0x00;
    if(line == 2)
        line_offset = 0x40;
    if(line == 3)
        line_offset = 0x14;
    if(line == 4)
        line_offset = 0x54;
    
    __delay_us(50);
    // set position
    LCD_RS_LAT = 0;
    LCD_E_LAT = 0;
    __delay_us(50);
    LATD &= 0xFFF0;
    LATD |= 0xF & (0x8 | ((line_offset + pos) >> 4));   // set position
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;
    __delay_us(50);
    LATD &= 0xFFF0;
    LATD |= 0xF & (line_offset + pos);   // set position
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;    
    __delay_us(50);
    
    //write character    
    LCD_RS_LAT = 1; // 1-data
    __delay_us(50);
    LATD &= 0xFFF0;
    LATD |= 0xF & (character >> 4);   // set character
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;
    __delay_us(50);
    LATD &= 0xFFF0;
    LATD |= 0xF & character;   // set character
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;
    __delay_us(50);
}

void lcd_clear(void)
{
    lcd_clear_line(1);
    lcd_clear_line(2);
    lcd_clear_line(3);
    lcd_clear_line(4);
}

void lcd_clear_line(int line)
{
    int i_lcd;
    lcd_send_pos(line, 0); //set position of the first character
    for(i_lcd = 0; i_lcd < LCD_LENGTH; ++i_lcd)//clear rest of characters
        lcd_send_next_char(' ');
}

void lcd_send_pos(int line, int pos)
{    
    int line_offset = 0;
    
    if(line == 1)
        line_offset = 0x00;
    if(line == 2)
        line_offset = 0x40;
    if(line == 3)
        line_offset = 0x14;
    if(line == 4)
        line_offset = 0x54;
    
    // set position
    LCD_RS_LAT = 0;
    LCD_E_LAT = 0;
    __delay_us(50);
    LATD &= 0xFFF0;
    LATD |= 0xF & (0x8 | ((line_offset + pos) >> 4));   // set position
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;
    __delay_us(50);
    LATD &= 0xFFF0;
    LATD |= 0xF & (line_offset + pos);   // set position
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;    
    __delay_us(50);
}

void lcd_send_next_char(char character)
{
    //write character    
    LCD_RS_LAT = 1; // 1-data
    LCD_E_LAT = 0;
    __delay_us(50);
    LATD &= 0xFFF0;
    LATD |= 0xF & (character >> 4);   // set character
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;
    __delay_us(50);
    LATD &= 0xFFF0;
    LATD |= 0xF & character;   // set character
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;
    __delay_us(50);
}

void lcd_send_instr(int instr)
{
    LCD_RS_LAT = 0;
    LCD_E_LAT = 0;
    __delay_us(50);
    LATD &= 0xFFF0;
    LATD |= 0x0F & (instr >> 4);   // set instruction
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;
    __delay_us(50);
    
    LATD &= 0xFFF0;
    LATD |= 0xF & instr;   // set instruction
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;
    __delay_us(50);
    
    if(((0xFF & instr) == 0x01) || ((0xFE & instr) == 0x02))
        __delay_ms(2);
}

void lcd_reset(void)
{
    __delay_ms(20);

    LCD_RS_LAT = 0;
    LCD_E_LAT = 0;
    __delay_us(50);
    LATD &= 0xFFF0;
    LATD |= 0x03;
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;
    __delay_ms(5);

    LCD_RS_LAT = 0;
    LCD_E_LAT = 0;
    __delay_us(50);
    LATD &= 0xFFF0;
    LATD |= 0x03;
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;
    __delay_us(50);
    __delay_us(50);
    __delay_us(50);

    LCD_RS_LAT = 0;
    LCD_E_LAT = 0;
    __delay_us(50);
    LATD &= 0xFFF0;
    LATD |= 0x03;
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;
    __delay_us(50);

    LCD_RS_LAT = 0;
    LCD_E_LAT = 0;
    __delay_us(50);
    LATD &= 0xFFF0;
    LATD |= 0x02;
    __delay_us(50);
    LCD_E_LAT = 1;
    __delay_us(50);
    LCD_E_LAT = 0;
    __delay_us(50);
}

void lcd_displ_line(int line, char *string)
{
    int i_eol;
    lcd_clear_line(line);
    for(i_eol = 0; string[i_eol] != '\0'; ++i_eol)
        ;
    lcd_send_string(string, i_eol, line, 0);
}

void lcd_param_and_val(char *param_string, char type, int line, void *arg)
{
    char lcd_buf[40];
    int i_lcd;
        
    switch(type)
    {
        case 'd':
            lcd_clear_line(line);
            sprintf(lcd_buf, param_string, *((int *)arg));
            for(i_lcd = 0; (lcd_buf[i_lcd] != '\0') && (i_lcd < 20); ++i_lcd)
                lcd_send_char(lcd_buf[i_lcd], line, i_lcd);
            break;
        case 'f':
            lcd_clear_line(line);
            sprintf(lcd_buf, param_string, *((double *)arg));
            for(i_lcd = 0; (lcd_buf[i_lcd] != '\0') && (i_lcd < 20); ++i_lcd)
                lcd_send_char(lcd_buf[i_lcd], line, i_lcd);
            break;
    }
}