/*
 * File:   ui.c
 * Author: michal
 *
 * Created on 18 września 2016, 23:45
 */


#include "ui.h"
#include "xc.h"
#include "ports.h"
#include "fpga_interface.h"
#include "flash.h"
#include "lcd.h"
#include <stdint.h>
#include <math.h>
#include <stdio.h>

extern volatile int g_key_numb;
extern int led_matrix[LED_SIZE_X * LED_SIZE_Y];

extern struct analog_cal_struct analog_cal[2];
extern struct wave_param_struct wave_param[2];
extern struct fpga_params_struct fpga_params;


void __attribute__((__interrupt__, no_auto_psv)) _T1Interrupt(void) //TODO - hardcoded ports
{
    // Read keyboard
    int key_numb = 0;
    static int last_key_numb = 0;
    int i_keybx;    //keyboard x coordinate
    int i_keyby;    //keyboard y coordinate
    int valid_cnt = 0;  //check if only one key is detected
    static int debounce_cnt;    //counter for tact-switches debouncing
    static int i_ledx = 0;    //LED x coordinate
    static int i_ledy = 0;    //LED y coordinate

    for(i_keybx = 0; i_keybx < KEYB_SIZE_X; ++i_keybx)
    {
        LATB |= 1 << (10 + i_keybx);
        for(i_keyby = 0; i_keyby < KEYB_SIZE_Y; ++i_keyby)
        {
            if(PORTB & (1 << (4 + i_keyby)))
            {
                key_numb = i_keybx * KEYB_SIZE_X + i_keyby;
                ++valid_cnt;
            }
        }
        LATB &= ~(1 << (10 + i_keybx));
    }
    
    if(valid_cnt == 1)
    {
        if(key_numb == last_key_numb)
            ++debounce_cnt;
        last_key_numb = key_numb;
        
        if(debounce_cnt > DEBOUNCE_CHECKS)
        {
            debounce_cnt = 0;
            g_key_numb = key_numb;    // it's the code of the key
        }
    }
    else
    {
        debounce_cnt = 0;
        g_key_numb = -1;          // nothing pressed or multiple keys pressed
    }
    
    //Update multiplexed diode matrix
    if(i_ledx < LED_SIZE_X)
        ++i_ledx;
    else
    {
        i_ledx = 0;
        if(i_ledy < LED_SIZE_Y)
            ++i_ledy;
        else
            i_ledy = 0;
    }
    LATG &= ~0xF000;    //turn off LEDs...
    LATG |= 0x000F;    //turn off LEDs...
    if(led_matrix[i_ledx * LED_SIZE_X + i_ledy])    //...and turn on one LED in this interruption if it's set as active in LEDs array
    {
        LATG |= 1 << (12 + i_ledx);
        LATG &= ~(1 << (0 + i_ledy));
    }
    
    //check if input overloaded
    if(AUX1_PORT == 1)
        led_matrix[LED_OV] = 1;
    else
        led_matrix[LED_OV] = 0;
    
    IFS0bits.T1IF = 0;  //clear interrupt flag
}




void menu_main()
{
    int channel = 0;    //number of selected channel, 0 or 1
    int context_position = 1;   //number of current screen in context menu (one menu is one or more screens)
    int context_screens = 2;    //number of all context menu screens available for selected function
    int locked = 0; // key press one time executes one action, this is set when button pressed and reset after releasing the button
    int refresh_lock = 0;   //variable that is set if screen in main menu needs refreshing
    static int last_mod_type = 1;  //to switch on/off modulation without selecting from list - initially sinus
    int key_freeze; //key number that was active while entering switch statement
    
    lcd_clear();
    lcd_displ_line(1, "wait");
    refresh_fpga_memory(1, 0);
    refresh_fpga_memory(1, 1);
    refresh_configuration_all(&fpga_params);
    led_matrix[LED_CH1] = 1;
    led_matrix[LED_FIX_F] = 1;
    led_matrix[LED_VAR_F] = 0;
    
    
    while(1)
    {
        if(refresh_lock == 0)   //if get in the main menu first time or back from submenu then refresh screen
        {
            lcd_clear();    
            switch(wave_param[channel].function)
            {
                case(FUNCT_SIN):
                    if(context_position == 1)
                        lcd_param_and_val("freq   = %f", 'f', 1, (void *)&wave_param[channel].freqency);
                    else if(context_position == 2)
                        lcd_param_and_val("ampl   = %f", 'f', 1, (void *)&wave_param[channel].amplitude);
                    if(context_position == 1)
                        lcd_param_and_val("phase  = %f", 'f', 2, (void *)&wave_param[channel].phase);
                    else if(context_position == 2)
                        lcd_param_and_val("offs   = %f", 'f', 2, (void *)&wave_param[channel].offset);
                    if(context_position == 2)
                        wave_param[channel].atten20db ? lcd_displ_line(3, "atten 20dB") : lcd_displ_line(3, "atten  0dB");
                    break;
                case(FUNCT_RECT):
                    if(context_position == 1)
                        lcd_param_and_val("freq   = %f", 'f', 1, (void *)&wave_param[channel].freqency);
                    else if(context_position == 2)
                        lcd_param_and_val("ampl   = %f", 'f', 1, (void *)&wave_param[channel].amplitude);
                    if(context_position == 1)
                        lcd_param_and_val("phase  = %f", 'f', 2, (void *)&wave_param[channel].phase);
                    else if(context_position == 2)
                        lcd_param_and_val("offs   = %f", 'f', 2, (void *)&wave_param[channel].offset);
                    if(context_position == 1)
                        lcd_param_and_val("symm   = %f", 'f', 3, (void *)&wave_param[channel].symmetry);
                    else if(context_position == 2)
                        wave_param[channel].atten20db ? lcd_displ_line(3, "atten 20dB") : lcd_displ_line(3, "atten  0dB");
                    break;
                case(FUNCT_TRI):
                    if(context_position == 1)
                        lcd_param_and_val("freq   = %f", 'f', 1, (void *)&wave_param[channel].freqency);
                    else if(context_position == 2)
                        lcd_param_and_val("ampl   = %f", 'f', 1, (void *)&wave_param[channel].amplitude);
                    if(context_position == 1)
                        lcd_param_and_val("phase  = %f", 'f', 2, (void *)&wave_param[channel].phase);
                    else if(context_position == 2)
                        lcd_param_and_val("offs   = %f", 'f', 2, (void *)&wave_param[channel].offset);
                    if(context_position == 1)
                        lcd_param_and_val("symm   = %f", 'f', 3, (void *)&wave_param[channel].symmetry);
                    else if(context_position == 2)
                        wave_param[channel].atten20db ? lcd_displ_line(3, "atten 20dB") : lcd_displ_line(3, "atten  0dB");
                    break;
                case(FUNCT_NOISE):
                    if(context_position == 1)
                        lcd_param_and_val("ampl   = %f", 'f', 1, (void *)&wave_param[channel].amplitude);
                    if(context_position == 1)
                        lcd_param_and_val("offs   = %f", 'f', 2, (void *)&wave_param[channel].offset);
                    if(context_position == 1)
                        wave_param[channel].atten20db ? lcd_displ_line(3, "atten 20dB") : lcd_displ_line(3, "atten  0dB");
                    break;
                case(FUNCT_ARB):
                    if(context_position == 1)
                        lcd_param_and_val("freq   = %f", 'f', 1, (void *)&wave_param[channel].freqency);
                    else if(context_position == 2)
                        lcd_param_and_val("ampl   = %f", 'f', 1, (void *)&wave_param[channel].amplitude);
                    if(context_position == 1)
                        lcd_param_and_val("phase  = %f", 'f', 2, (void *)&wave_param[channel].phase);
                    else if(context_position == 2)
                        lcd_param_and_val("offs   = %f", 'f', 2, (void *)&wave_param[channel].offset);
                    if(context_position == 1)
                        lcd_param_and_val("wave   = %d", 'd', 3, (void *)&wave_param[channel].wave_num);
                    else if(context_position == 2)
                        wave_param[channel].atten20db ? lcd_displ_line(3, "atten 20dB") : lcd_displ_line(3, "atten  0dB");
                    break;
            }
            led_funct_set(wave_param[channel].function);

            refresh_lock = 1;
        }
        
        
        switch(key_freeze = g_key_numb)  //cases to execute if tact-switch pressed
        {
            case KEY_CONTEXT_1:
                if(locked == 0)
                {
                    switch(wave_param[channel].function)
                    {
                        case(FUNCT_SIN):
                            if(context_position == 1)
                            {
                                knob(&wave_param[channel].freqency, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].freqency = enter_float(wave_param[channel].freqency, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("freq   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].freqency);
                            }
                            else if(context_position == 2)
                            {
                                knob(&wave_param[channel].amplitude, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].amplitude = enter_float(wave_param[channel].amplitude, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("ampl   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].amplitude);
                            }
                            refresh_wave(channel, &fpga_params);
                            send_configuration_fpga(fpga_params, channel);
                            break;
                        case(FUNCT_RECT):
                            if(context_position == 1)
                            {
                                knob(&wave_param[channel].freqency, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].freqency = enter_float(wave_param[channel].freqency, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("freq   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].freqency);
                            }
                            else if(context_position == 2)
                            {
                                knob(&wave_param[channel].amplitude, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].amplitude = enter_float(wave_param[channel].amplitude, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("ampl   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].amplitude);
                            }
                            refresh_wave(channel, &fpga_params);
                            send_configuration_fpga(fpga_params, channel);
                            break;
                        case(FUNCT_TRI):
                            if (context_position == 1)
                            {
                                knob(&wave_param[channel].freqency, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].freqency = enter_float(wave_param[channel].freqency, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("freq   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].freqency);
                            }
                            else if (context_position == 2)
                            {
                                knob(&wave_param[channel].amplitude, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].amplitude = enter_float(wave_param[channel].amplitude, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("ampl   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].amplitude);
                            }
                            refresh_wave(channel, &fpga_params);
                            send_configuration_fpga(fpga_params, channel);
                            break;
                        case(FUNCT_NOISE):
                            if (context_position == 1)
                            {
                                knob(&wave_param[channel].amplitude, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].amplitude = enter_float(wave_param[channel].amplitude, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("ampl   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].amplitude);
                            }
                            refresh_wave(channel, &fpga_params);
                            send_configuration_fpga(fpga_params, channel);
                            break;
                        case(FUNCT_ARB):
                            if (context_position == 1)
                            {
                                knob(&wave_param[channel].freqency, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].freqency = enter_float(wave_param[channel].freqency, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("freq   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].freqency);
                            }
                            else if (context_position == 2)
                            {
                                knob(&wave_param[channel].amplitude, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].amplitude = enter_float(wave_param[channel].amplitude, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("ampl   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].amplitude);
                            }
                            refresh_wave(channel, &fpga_params);
                            send_configuration_fpga(fpga_params, channel);
                            break;
                    }
                }
                locked = 1;
                break;
            
            case KEY_CONTEXT_2:
                if(locked == 0)
                {
                    switch(wave_param[channel].function)
                    {
                        case(FUNCT_SIN):
                            if(context_position == 1)
                            {
                                knob(&wave_param[channel].phase, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].phase = enter_float(wave_param[channel].phase, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("phase  = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].phase);
                            }
                            else if(context_position == 2)
                            {
                                knob(&wave_param[channel].offset, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].offset = enter_float(wave_param[channel].offset, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("offs   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].offset);
                            }
                            refresh_wave(channel, &fpga_params);
                            send_configuration_fpga(fpga_params, channel);
                            break;
                        case(FUNCT_RECT):
                            if(context_position == 1)
                            {
                                knob(&wave_param[channel].phase, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].phase = enter_float(wave_param[channel].phase, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("phase  = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].phase);
                            }
                            else if(context_position == 2)
                            {
                                knob(&wave_param[channel].offset, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].offset = enter_float(wave_param[channel].offset, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("offs   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].offset);
                            }
                            refresh_wave(channel, &fpga_params);
                            send_configuration_fpga(fpga_params, channel);
                            break;
                        case(FUNCT_TRI):
                            if(context_position == 1)
                            {
                                knob(&wave_param[channel].phase, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].phase = enter_float(wave_param[channel].phase, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("phase  = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].phase);
                            }
                            else if(context_position == 2)
                            {
                                knob(&wave_param[channel].offset, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].offset = enter_float(wave_param[channel].offset, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("offs   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].offset);
                            }
                            refresh_wave(channel, &fpga_params);
                            send_configuration_fpga(fpga_params, channel);
                            break;
                        case(FUNCT_NOISE):
                            if(context_position == 1)
                            {
                                knob(&wave_param[channel].offset, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].offset = enter_float(wave_param[channel].offset, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("offs   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].offset);
                                refresh_wave(channel, &fpga_params);
                                send_configuration_fpga(fpga_params, channel);
                            }
                            break;
                        case(FUNCT_ARB):
                            if(context_position == 1)
                            {
                                knob(&wave_param[channel].phase, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].phase = enter_float(wave_param[channel].phase, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("phase  = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].phase);
                            }
                            else if(context_position == 2)
                            {
                                knob(&wave_param[channel].offset, CONT_LINE_NUM(key_freeze), channel);
                                if(KEY_ONE_OF_NUMERIC_KEYS)
                                    wave_param[channel].offset = enter_float(wave_param[channel].offset, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("offs   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].offset);
                            }
                            refresh_wave(channel, &fpga_params);
                            send_configuration_fpga(fpga_params, channel);
                            break;
                    }
                }
                locked = 1;
                break;

            case KEY_CONTEXT_3:
                if (locked == 0)
                {
                    switch (wave_param[channel].function)
                    {
                        case(FUNCT_SIN):
                            if (context_position == 2)
                            {
                                if(wave_param[channel].atten20db == 0)
                                {
                                    wave_param[channel].atten20db = 1;
                                    lcd_displ_line(CONT_LINE_NUM(key_freeze), "atten 20dB");
                                    if(channel == 0)
                                        OUT_ATTEN_DIS_A_LAT = 0;
                                    else if(channel == 1)
                                        OUT_ATTEN_DIS_B_LAT = 0;
                                }
                                else
                                {
                                    wave_param[channel].atten20db = 0;
                                    lcd_displ_line(CONT_LINE_NUM(key_freeze), "atten  0dB");
                                    if(channel == 0)
                                        OUT_ATTEN_DIS_A_LAT = 1;
                                    else if(channel == 1)
                                        OUT_ATTEN_DIS_B_LAT = 1;
                                }
                            }
                            break;
                        case(FUNCT_RECT):
                            if (context_position == 1)
                            {
                                wave_param[channel].symmetry = enter_float(wave_param[channel].symmetry, CONT_LINE_NUM(key_freeze));
                                refresh_wave(channel, &fpga_params);
                                lcd_displ_line(CONT_LINE_NUM(key_freeze), "wait");
                                generate_rect_memory(wave_param[channel], channel); //for rectangle calculate and send samples to FPGA instead of copying it from flash - this should allow achieving better precision of pulses when variable clock dds is used
                                send_configuration_fpga(fpga_params, channel);
                                lcd_param_and_val("symm   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].symmetry);
                            }
                            else if (context_position == 2)
                            {
                                if(wave_param[channel].atten20db == 0)
                                {
                                    wave_param[channel].atten20db = 1;
                                    lcd_displ_line(CONT_LINE_NUM(key_freeze), "atten 20dB");
                                    if(channel == 0)
                                        OUT_ATTEN_DIS_A_LAT = 0;
                                    else if(channel == 1)
                                        OUT_ATTEN_DIS_B_LAT = 0;
                                }
                                else
                                {
                                    wave_param[channel].atten20db = 0;
                                    lcd_displ_line(CONT_LINE_NUM(key_freeze), "atten  0dB");
                                    if(channel == 0)
                                        OUT_ATTEN_DIS_A_LAT = 1;
                                    else if(channel == 1)
                                        OUT_ATTEN_DIS_B_LAT = 1;
                                }
                            }
                            break;
                        case(FUNCT_TRI):
                            if (context_position == 1)
                            {
                                wave_param[channel].symmetry = enter_float(wave_param[channel].symmetry, CONT_LINE_NUM(key_freeze));
                                refresh_wave(channel, &fpga_params);
                                send_configuration_fpga(fpga_params, channel);
                                lcd_displ_line(CONT_LINE_NUM(key_freeze), "wait");
                                generate_tri_memory(wave_param[channel], channel);
                                lcd_param_and_val("symm   = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].symmetry);
                            }
                            else if (context_position == 2)
                            {
                                if(wave_param[channel].atten20db == 0)
                                {
                                    wave_param[channel].atten20db = 1;
                                    lcd_displ_line(CONT_LINE_NUM(key_freeze), "atten 20dB");
                                    if(channel == 0)
                                        OUT_ATTEN_DIS_A_LAT = 0;
                                    else if(channel == 1)
                                        OUT_ATTEN_DIS_B_LAT = 0;
                                }
                                else
                                {
                                    wave_param[channel].atten20db = 0;
                                    lcd_displ_line(CONT_LINE_NUM(key_freeze), "atten  0dB");
                                    if(channel == 0)
                                        OUT_ATTEN_DIS_A_LAT = 1;
                                    else if(channel == 1)
                                        OUT_ATTEN_DIS_B_LAT = 1;
                                }
                            }
                            break;
                        case(FUNCT_NOISE):
                            if (context_position == 1)
                            {
                                if(wave_param[channel].atten20db == 0)
                                {
                                    wave_param[channel].atten20db = 1;
                                    lcd_displ_line(CONT_LINE_NUM(key_freeze), "atten 20dB");
                                    if(channel == 0)
                                        OUT_ATTEN_DIS_A_LAT = 0;
                                    else if(channel == 1)
                                        OUT_ATTEN_DIS_B_LAT = 0;
                                }
                                else
                                {
                                    wave_param[channel].atten20db = 0;
                                    lcd_displ_line(CONT_LINE_NUM(key_freeze), "atten  0dB");
                                    if(channel == 0)
                                        OUT_ATTEN_DIS_A_LAT = 1;
                                    else if(channel == 1)
                                        OUT_ATTEN_DIS_B_LAT = 1;
                                }
                            }
                            break;
                        case(FUNCT_ARB):
                            if (context_position == 1)
                            {
                                wave_param[channel].wave_num = enter_uint(wave_param[channel].wave_num, CONT_LINE_NUM(key_freeze));
                                lcd_param_and_val("wave   = %d", 'd', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].wave_num);
                                refresh_wave(channel, &fpga_params);
                                send_configuration_fpga(fpga_params, channel);
                                refresh_fpga_memory(wave_param[channel].wave_num, channel);
                            }
                            else if (context_position == 2)
                            {
                                if(wave_param[channel].atten20db == 0)
                                {
                                    wave_param[channel].atten20db = 1;
                                    lcd_displ_line(CONT_LINE_NUM(key_freeze), "atten 20dB");
                                    if(channel == 0)
                                        OUT_ATTEN_DIS_A_LAT = 0;
                                    else if(channel == 1)
                                        OUT_ATTEN_DIS_B_LAT = 0;
                                }
                                else
                                {
                                    wave_param[channel].atten20db = 0;
                                    lcd_displ_line(CONT_LINE_NUM(key_freeze), "atten  0dB");
                                    if(channel == 0)
                                        OUT_ATTEN_DIS_A_LAT = 1;
                                    else if(channel == 1)
                                        OUT_ATTEN_DIS_B_LAT = 1;
                                }
                            }
                            break;
                    }
                }
                locked = 1;
                break;

            case KEY_CONTEXT_4:
                if(locked == 0)
                {
                    switch(wave_param[channel].function)
                    {
                        case(FUNCT_SIN):
                            break;
                        case(FUNCT_RECT):
                            break;
                        case(FUNCT_TRI):
                            break;
                        case(FUNCT_NOISE):
                            break;
                        case(FUNCT_ARB):
                            break;
                    }
                }
                locked = 1;
                break;
                
            case KEY_CONTEXT_UP:
                if(locked == 0)
                {
                    if(context_position > 1)
                    {
                        --context_position;
                        refresh_lock = 0;
                    }
                }
                locked = 1;
                break;
            
            case KEY_CONTEXT_DOWN:
                if(locked == 0)
                {
                    if(context_position < context_screens)
                    {
                        ++context_position;
                        refresh_lock = 0;
                    }
                }
                locked = 1;
                break;
            
            case KEY_SELECT_CHANNEL_1:
                if(locked == 0)
                {
                    channel = 0;
                    led_matrix[LED_CH2] = 0;
                    led_matrix[LED_CH1] = 1;
                    refresh_lock = 0;
                }
                locked = 1;
                break;
            
            case KEY_SELECT_CHANNEL_2:
                if(locked == 0)
                {
                    channel = 1;
                    led_matrix[LED_CH2] = 1;
                    led_matrix[LED_CH1] = 0;
                    refresh_lock = 0;
                }
                locked = 1;
                break;
                
            case KEY_FUNCT_SIN:
                if(locked == 0)
                {
                    lcd_clear();
                    lcd_displ_line(1, "wait");
                    wave_param[channel].function = FUNCT_SIN;
                    context_position = 1;
                    context_screens = 2;
                    refresh_wave(channel, &fpga_params);
                    refresh_fpga_memory(1, channel);
                    send_configuration_fpga(fpga_params, channel);
                    refresh_lock = 0;
                }
                locked = 1;
                break;
            
            case KEY_FUNCT_RECT:
                if(locked == 0)
                {
                    lcd_clear();
                    lcd_displ_line(1, "wait");
                    wave_param[channel].function = FUNCT_RECT;
                    context_position = 1;
                    context_screens = 2;
                    refresh_wave(channel, &fpga_params);
                    generate_rect_memory(wave_param[channel], channel);
                    send_configuration_fpga(fpga_params, channel);
                    refresh_lock = 0;
                }
                locked = 1;
                break;
            
            case KEY_FUNCT_TRI:
                if(locked == 0)
                {
                    lcd_clear();
                    lcd_displ_line(1, "wait");
                    wave_param[channel].function = FUNCT_TRI;
                    context_position = 1;
                    context_screens = 2;
                    refresh_wave(channel, &fpga_params);
                    generate_tri_memory(wave_param[channel], channel);
                    send_configuration_fpga(fpga_params, channel);
                    refresh_lock = 0;
                }
                locked = 1;
                break;

            case KEY_FUNCT_NOISE:
                if(locked == 0)
                {
                    lcd_clear();
                    lcd_displ_line(1, "wait");
                    wave_param[channel].function = FUNCT_NOISE;
                    context_position = 1;
                    context_screens = 1;
                    refresh_wave(channel, &fpga_params);
                    refresh_fpga_memory(4, channel);
                    send_configuration_fpga(fpga_params, channel);
                    refresh_lock = 0;
                }
                locked = 1;
                break;
            
            case KEY_FUNCT_ARB:
                if(locked == 0)
                {
                    lcd_clear();
                    lcd_displ_line(1, "wait");
                    wave_param[channel].function = FUNCT_ARB;
                    context_position = 1;
                    context_screens = 2;
                    refresh_wave(channel, &fpga_params);
                    refresh_fpga_memory(wave_param[channel].wave_num, channel);
                    send_configuration_fpga(fpga_params, channel);
                    refresh_lock = 0;
                }
                locked = 1;
                break;
            
            case KEY_CH1_ON:
                if(locked == 0)
                {
                    if(OUT_EN_A_LAT == 0)
                    {
                        OUT_EN_A_LAT = 1;
                        led_matrix[LED_CH1_ON] = 1;
                    }
                    else
                    {
                        OUT_EN_A_LAT = 0;
                        led_matrix[LED_CH1_ON] = 0;
                    }
                }
                locked = 1;
                break;
            
            case KEY_CH2_ON:
                if(locked == 0)
                {
                    if(OUT_EN_B_LAT == 0)
                    {
                        OUT_EN_B_LAT = 1;
                        led_matrix[LED_CH2_ON] = 1;
                    }
                    else
                    {
                        OUT_EN_B_LAT = 0;
                        led_matrix[LED_CH2_ON] = 0;
                    }
                }
                locked = 1;
                break;
            
            case KEY_UTILS:
                if(locked == 0)
                {
                    menu_utils(&fpga_params);
                    refresh_lock = 0;
                }
                locked = 1;
                break;
                
            case KEY_MOD_SET:
            {
                if(locked == 0)
                {
                    menu_mod(channel, wave_param, &fpga_params);
                    refresh_lock = 0;
                }
                locked = 1;
                break;
            }
            
            case KEY_MOD_ON:
                if(locked == 0)
                {
                    if(!wave_param[channel].mod_type)
                    {
                        wave_param[channel].mod_type = last_mod_type;
                        led_matrix[LED_MOD_ON] = 1;
                    }
                    else
                    {
                        last_mod_type = wave_param[channel].mod_type;
                        wave_param[channel].mod_type = MOD_NONE;
                        led_matrix[LED_MOD_ON] = 0;
                    }
                    refresh_wave(channel, &fpga_params);
                    send_configuration_fpga(fpga_params, channel);
                }
                locked = 1;
                break;
            
            case -1:
                locked = 0;
                break;
            
        }
    }
}

void menu_utils(struct fpga_params_struct *p_fpga_params)
{
    int locked = 1;
    int refresh_lock = 0;
    
    while(1)
    {
        if(refresh_lock == 0)
        {
            lcd_clear();    
            lcd_send_string("calibration menu", 16, 1, 0);
            if(fpga_params.dds_mode == 0)
            {
                lcd_send_string("dds mode = var freq", 19, 2, 0);
                led_matrix[LED_FIX_F] = 0;
                led_matrix[LED_VAR_F] = 1;
            }
            else
            {
                lcd_send_string("dds mode = fix freq", 19, 2, 0);
                led_matrix[LED_FIX_F] = 1;
                led_matrix[LED_VAR_F] = 0;
            }

            switch (fpga_params.trig_sel)
            {
                case TRIG_NONE:
                    lcd_send_string("trig = NONE", 11, 3, 0);
                    break;
                case TRIG_WAV1:
                    lcd_send_string("trig = WAV1", 11, 3, 0);
                    break;
                case TRIG_WAV2:
                    lcd_send_string("trig = WAV2", 11, 3, 0);
                    break;
                case TRIG_MOD1:
                    lcd_send_string("trig = MOD1", 11, 3, 0);
                    break;
                case TRIG_MOD2:
                    lcd_send_string("trig = MOD2", 11, 3, 0);
                    break;
            }
                                
            refresh_lock = 1;
        }
        
        switch(g_key_numb)
        {
            
            case KEY_CONTEXT_1:
                if(locked == 0)
                {
                    menu_cal(p_fpga_params);
                    refresh_lock = 0;
                }
                locked = 1;
                break;
            
            case KEY_CONTEXT_2:
                if(locked == 0)
                {
                    if(fpga_params.dds_mode == 0)
                    {
                        fpga_params.dds_mode = 1;
                        lcd_send_string("fix freq", 8, 2, 11);
                        refresh_configuration_all(p_fpga_params);
                    }
                    else
                    {
                        fpga_params.dds_mode = 0;
                        lcd_send_string("var freq", 8, 2, 11);
                        refresh_configuration_all(p_fpga_params);
                    }
                }
                locked = 1;
                break;
                
            case KEY_CONTEXT_3:
                if(locked == 0)
                {
                    ++fpga_params.trig_sel;
                    if(fpga_params.trig_sel > TRIG_NONE)
                        fpga_params.trig_sel = 0;
                    
                    switch (fpga_params.trig_sel)
                    {
                        case TRIG_WAV1:
                            lcd_send_string("trig = WAV1", 11, 3, 0);
                            break;
                        case TRIG_WAV2:
                            lcd_send_string("trig = WAV2", 11, 3, 0);
                            break;
                        case TRIG_MOD1:
                            lcd_send_string("trig = MOD1", 11, 3, 0);
                            break;
                        case TRIG_MOD2:
                            lcd_send_string("trig = MOD2", 11, 3, 0);
                            break;
                        case TRIG_NONE:
                            lcd_send_string("trig = NONE", 11, 3, 0);
                            break;
                    }
                    refresh_configuration_all(p_fpga_params);
                }
                locked = 1;
                break;
                
            case -1:
                locked = 0;
                break;
                
            case KEY_EXIT:
                locked = 1;
                return;
        }
    }
}

void menu_cal(struct fpga_params_struct *p_fpga_params)
{
    int locked = 1;
    int context_screens = 3;
    int context_position = 1;
    int channel = context_position - 1;
    int lcd_contrast;
    char lcd_buf[40];
    int i_lcd;
    int key_freeze;
    
    int32_t mem_address;
    int page_buf[256];
    int i_data;
    int32_t cal_buf[8];


    lcd_clear();
    sprintf(lcd_buf, "amp i%d = %f", channel, analog_cal[channel].in_amplitude);
    for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
        lcd_send_char(lcd_buf[i_lcd], 1, i_lcd);
    sprintf(lcd_buf, "off i%d = %f", channel, analog_cal[channel].in_offset);
    for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
        lcd_send_char(lcd_buf[i_lcd], 2, i_lcd);
    sprintf(lcd_buf, "amp o%d = %f", channel, analog_cal[channel].out_amplitude);
    for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
        lcd_send_char(lcd_buf[i_lcd], 3, i_lcd);
    sprintf(lcd_buf, "off o%d = %f", channel, analog_cal[channel].out_offset);
    for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
        lcd_send_char(lcd_buf[i_lcd], 4, i_lcd);
    
    while(1)
    {
        switch(key_freeze = g_key_numb)  //cases to execute if tact-switch pressed
        {
            
            case KEY_CONTEXT_1:
                if(locked == 0)
                {
                    if(context_position == 1)
                    {
                        knob(&analog_cal[channel].in_amplitude, CONT_LINE_NUM(key_freeze), channel);
                        if(KEY_ONE_OF_NUMERIC_KEYS)
                            analog_cal[channel].in_amplitude = enter_float(analog_cal[channel].in_amplitude, CONT_LINE_NUM(key_freeze));
                        lcd_param_and_val("amp i0 = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&analog_cal[channel].in_amplitude);
                    }
                    else if(context_position == 2)
                    {
                        knob(&analog_cal[channel].in_amplitude, CONT_LINE_NUM(key_freeze), channel);
                        if(KEY_ONE_OF_NUMERIC_KEYS)
                            analog_cal[channel].in_amplitude = enter_float(analog_cal[channel].in_amplitude, CONT_LINE_NUM(key_freeze));
                        lcd_param_and_val("amp i1 = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&analog_cal[channel].in_amplitude);
                    }
                    else if(context_position == 3)
                    {
                        OC5RS = (lcd_contrast = enter_uint(OC5RS >> 8, 1)) << 8;
                        sprintf(lcd_buf, "contr  = %d", lcd_contrast);
                        for(i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 1, i_lcd);
                    }
                    refresh_configuration_all(p_fpga_params);
                }
                locked = 1;
                break;
                
            case KEY_CONTEXT_2:
                if(locked == 0)
                {
                    if(context_position == 1)
                    {
                        knob(&analog_cal[channel].in_offset, CONT_LINE_NUM(key_freeze), channel);
                        if(KEY_ONE_OF_NUMERIC_KEYS)
                            analog_cal[channel].in_offset = enter_float(analog_cal[channel].in_offset, CONT_LINE_NUM(key_freeze));
                        lcd_param_and_val("off i0 = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&analog_cal[channel].in_offset);
                    }
                    else if(context_position == 2)
                    {
                        knob(&analog_cal[channel].in_offset, CONT_LINE_NUM(key_freeze), channel);
                        if(KEY_ONE_OF_NUMERIC_KEYS)
                            analog_cal[channel].in_offset = enter_float(analog_cal[channel].in_offset, CONT_LINE_NUM(key_freeze));
                        lcd_param_and_val("off i1 = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&analog_cal[channel].in_offset);
                    }                    
                    refresh_configuration_all(p_fpga_params);
                }
                locked = 1;
                break;
                
            case KEY_CONTEXT_3:
                if(locked == 0)
                {
                    if(context_position == 1)
                    {
                        knob(&analog_cal[channel].out_amplitude, CONT_LINE_NUM(key_freeze), channel);
                        if(KEY_ONE_OF_NUMERIC_KEYS)
                            analog_cal[channel].out_amplitude = enter_float(analog_cal[channel].out_amplitude, CONT_LINE_NUM(key_freeze));
                        lcd_param_and_val("amp o0 = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&analog_cal[channel].out_amplitude);
                    }
                    else if(context_position == 2)
                    {
                        knob(&analog_cal[channel].out_amplitude, CONT_LINE_NUM(key_freeze), channel);
                        if(KEY_ONE_OF_NUMERIC_KEYS)
                            analog_cal[channel].out_amplitude = enter_float(analog_cal[channel].out_amplitude, CONT_LINE_NUM(key_freeze));
                        lcd_param_and_val("amp o1 = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&analog_cal[channel].out_amplitude);
                    }
                    refresh_configuration_all(p_fpga_params);
                }
                locked = 1;
                break;
                
            case KEY_CONTEXT_4:
                if(locked == 0)
                {
                    if(context_position == 1)
                    {
                        knob(&analog_cal[channel].out_offset, CONT_LINE_NUM(key_freeze), channel);
                        if(KEY_ONE_OF_NUMERIC_KEYS)
                            analog_cal[channel].out_offset = enter_float(analog_cal[channel].out_offset, CONT_LINE_NUM(key_freeze));
                        lcd_param_and_val("off o0 = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&analog_cal[channel].out_offset);
                    }
                    else if(context_position == 2)
                    {
                        knob(&analog_cal[channel].out_offset, CONT_LINE_NUM(key_freeze), channel);
                        if(KEY_ONE_OF_NUMERIC_KEYS)
                            analog_cal[channel].out_offset = enter_float(analog_cal[channel].out_offset, CONT_LINE_NUM(key_freeze));
                        lcd_param_and_val("off o1 = %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&analog_cal[channel].out_offset);
                    }
                    refresh_configuration_all(p_fpga_params);
                }
                locked = 1;
                break;
                
            case KEY_CONTEXT_UP:
                if(locked == 0)
                {
                    if(context_position > 1)
                        --context_position;
                    channel = context_position - 1;
                    
                    if(context_position == 1)
                    {
                        lcd_clear();
                        sprintf(lcd_buf, "amp i%d = %f", channel, analog_cal[channel].in_amplitude);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 1, i_lcd);
                        sprintf(lcd_buf, "off i%d = %f", channel, analog_cal[channel].in_offset);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 2, i_lcd);
                        sprintf(lcd_buf, "amp o%d = %f", channel, analog_cal[channel].out_amplitude);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 3, i_lcd);
                        sprintf(lcd_buf, "off o%d = %f", channel, analog_cal[channel].out_offset);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 4, i_lcd);
                    }
                    else if(context_position == 2)
                    {
                        lcd_clear();
                        sprintf(lcd_buf, "amp i%d = %f", channel, analog_cal[channel].in_amplitude);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 1, i_lcd);
                        sprintf(lcd_buf, "off i%d = %f", channel, analog_cal[channel].in_offset);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 2, i_lcd);
                        sprintf(lcd_buf, "amp o%d = %f", channel, analog_cal[channel].out_amplitude);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 3, i_lcd);
                        sprintf(lcd_buf, "off o%d = %f", channel, analog_cal[channel].out_offset);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 4, i_lcd);
                    }
                    else if(context_position == 3)
                    {
                        lcd_clear();
                        sprintf(lcd_buf, "contr  = %d", OC5RS >> 8);
                        for(i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 1, i_lcd);
                    }
                }
                locked = 1;
                break;
            
            case KEY_CONTEXT_DOWN:
                if(locked == 0)
                {
                    if(context_position < context_screens)
                        ++context_position;
                    channel = context_position - 1;

                    if(context_position == 1)
                    {
                        lcd_clear();
                        sprintf(lcd_buf, "amp i%d = %f", channel, analog_cal[channel].in_amplitude);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 1, i_lcd);
                        sprintf(lcd_buf, "off i%d = %f", channel, analog_cal[channel].in_offset);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 2, i_lcd);
                        sprintf(lcd_buf, "amp o%d = %f", channel, analog_cal[channel].out_amplitude);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 3, i_lcd);
                        sprintf(lcd_buf, "off o%d = %f", channel, analog_cal[channel].out_offset);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 4, i_lcd);
                    }
                    else if(context_position == 2)
                    {
                        lcd_clear();
                        sprintf(lcd_buf, "amp i%d = %f", channel, analog_cal[channel].in_amplitude);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 1, i_lcd);
                        sprintf(lcd_buf, "off i%d = %f", channel, analog_cal[channel].in_offset);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 2, i_lcd);
                        sprintf(lcd_buf, "amp o%d = %f", channel, analog_cal[channel].out_amplitude);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 3, i_lcd);
                        sprintf(lcd_buf, "off o%d = %f", channel, analog_cal[channel].out_offset);
                        for (i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 4, i_lcd);
                    }
                    else if(context_position == 3)
                    {
                        lcd_clear();
                        sprintf(lcd_buf, "contr  = %d", OC5RS >> 8);
                        for(i_lcd = 0; lcd_buf[i_lcd] != '\0'; ++i_lcd)
                            lcd_send_char(lcd_buf[i_lcd], 1, i_lcd);
                    }
                }
                locked = 1;
                break;
                                
            case KEY_EXIT:
                if(locked == 0)
                {
                    mem_address = 0x07F000;

                    cal_buf[0] = *((int32_t *)&analog_cal[0].in_amplitude); //save 32-bit IEEE-754 representation of floating point number as integer
                    cal_buf[1] = *((int32_t *)&analog_cal[0].in_offset);
                    cal_buf[2] = *((int32_t *)&analog_cal[0].out_amplitude);
                    cal_buf[3] = *((int32_t *)&analog_cal[0].out_offset);
                    cal_buf[4] = *((int32_t *)&analog_cal[1].in_amplitude);
                    cal_buf[5] = *((int32_t *)&analog_cal[1].in_offset);
                    cal_buf[6] = *((int32_t *)&analog_cal[1].out_amplitude);
                    cal_buf[7] = *((int32_t *)&analog_cal[1].out_offset);
                    for(i_data = 0; i_data < 8; ++i_data)   //save 8 words (32 bytes) of calibration data
                    {
                        page_buf[i_data * 4 + 3] = 0xFF & (cal_buf[i_data] >> 24);
                        page_buf[i_data * 4 + 2] = 0xFF & (cal_buf[i_data] >> 16);
                        page_buf[i_data * 4 + 1] = 0xFF & (cal_buf[i_data] >> 8);
                        page_buf[i_data * 4 + 0] = 0xFF & (cal_buf[i_data] >> 0);
                    }

                    page_buf[i_data * 4] = lcd_contrast; //...and one byte to set LCD contrast

                    mem_erase_4k(mem_address);
                    mem_write_page(page_buf, mem_address);
                    locked = 1;
                    return;
                }
                break;
                
            case -1:
                locked = 0;
                break;
        }
    }
}

void menu_mod(int channel, struct wave_param_struct *wave_param, struct fpga_params_struct *p_fpga_params)
{
    int locked = 1;
    static int last_mod_type = 1;  //to switch on/off modulation without selecting from list - initially sinus
    int context_position = 1;
    int context_screens = 2;
    int key_freeze;
    int uart_mod_start_byte = 0;
    
    lcd_clear();
    
    if (wave_param[channel].mod_type == MOD_NONE)
        lcd_send_string("modul = NONE ", 13, 1, 0);
    else if (wave_param[channel].mod_type == MOD_AM)
        lcd_send_string("modul = AM   ", 13, 1, 0);
    else if (wave_param[channel].mod_type == MOD_AM_SC)
        lcd_send_string("modul = AM-SC", 13, 1, 0);
    else if (wave_param[channel].mod_type == MOD_FM)
        lcd_send_string("modul = FM   ", 13, 1, 0);
    else if (wave_param[channel].mod_type == MOD_PM)
        lcd_send_string("modul = PM   ", 13, 1, 0);
    else if (wave_param[channel].mod_type == MOD_PWM)
        lcd_send_string("modul = PWM  ", 13, 1, 0);
    lcd_param_and_val("mod amp= %f", 'f', 2, (void *) &wave_param[channel].mod_ampl);
    lcd_param_and_val("mod freq=%f", 'f', 3, (void *) &wave_param[channel].mod_freq);
    lcd_param_and_val("mod dev= %f", 'f', 4, (void *) &wave_param[channel].mod_dev);
    
    while(1)
    {
        switch(key_freeze = g_key_numb)  //cases to execute if tact-switch pressed
        {
            case KEY_CONTEXT_1:
                if(locked == 0)
                {
                    if(context_position == 1)
                    {
                        ++wave_param[channel].mod_type;
                        if(wave_param[channel].mod_type == MOD_PWM)
                            if(wave_param[channel].function != FUNCT_TRI)
                                ++wave_param[channel].mod_type;
                        if(wave_param[channel].mod_type > MOD_PWM)
                            wave_param[channel].mod_type = 0;
                        switch (wave_param[channel].mod_type)
                        {
                            case MOD_NONE:
                                lcd_send_string("modul = NONE ", 13, 1, 0);
                                break;
                            case MOD_AM:
                                lcd_send_string("modul = AM   ", 13, 1, 0);
                                break;
                            case MOD_AM_SC:
                                lcd_send_string("modul = AM-SC", 13, 1, 0);
                                break;
                            case MOD_FM:
                                lcd_send_string("modul = FM   ", 13, 1, 0);
                                break;
                            case MOD_PM:
                                lcd_send_string("modul = PM   ", 13, 1, 0);
                                break;
                            case MOD_PWM:
                                lcd_send_string("modul = PWM  ", 13, 1, 0);
                                break;
                        }
                        if(wave_param[channel].mod_type == MOD_NONE)
                            led_matrix[LED_MOD_ON] = 0;
                        else
                            led_matrix[LED_MOD_ON] = 1;
                        refresh_wave(channel, p_fpga_params);
                        send_configuration_fpga(*p_fpga_params, channel);
                    }
                    else
                    {
                        lcd_send_string("waiting for data", 16, 1, 0);
                        wave_param[channel].mod_write_not_read = 1;
                        refresh_configuration_all(p_fpga_params);
                        IEC0bits.U1RXIE = 0;    // disable interrupt
                        while(((uart_mod_start_byte = U1RXREG) != START_BYTE) && (g_key_numb != KEY_EXIT));   //wait for start bit or exit key
                        IEC0bits.T1IE = 0;    // disable interrupt
                        if(uart_mod_start_byte == START_BYTE)
                        {
                            led_matrix[LED_BUSY] = 1;
                            modmem_write(uart_mod_start_byte);  //write wave from uart to memory (start bit and preamble to be checked inside the function)
                            led_matrix[LED_BUSY] = 0;
                        }
                        IEC0bits.U1RXIE = 1;    // enable interrupt
                        IEC0bits.T1IE = 1;    // enable interrupt
                        wave_param[channel].mod_write_not_read = 0;
                        refresh_configuration_all(p_fpga_params);
                        lcd_send_string("write modul. wave", 17, 1, 0);
                    }
                }
                locked = 1;
                break;

            case KEY_CONTEXT_2:
                if(locked == 0)
                {
                    if(context_position == 1)
                    {
                        knob(&wave_param[channel].mod_ampl, CONT_LINE_NUM(key_freeze), channel);
                        if(KEY_ONE_OF_NUMERIC_KEYS)
                            wave_param[channel].mod_ampl = enter_float(wave_param[channel].mod_ampl, CONT_LINE_NUM(key_freeze));
                        lcd_param_and_val("mod amp= %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].mod_ampl);
                    }
                    else    //select internal or external modulating signal
                    {
                        lcd_clear_line(2);
                        if((wave_param[channel].mod_ext_not_int) == 1 && (channel == 0))
                        {
                            wave_param[channel].mod_ext_not_int = 0;
                            lcd_send_string("mod int", 7, 2, 0);
                        }
                        else
                        {
                            wave_param[channel].mod_ext_not_int = 1;
                            lcd_send_string("mod ext", 7, 2, 0);
                        }
                    }
                    refresh_configuration_all(p_fpga_params);
                }
                locked = 1;
                break;

            case KEY_CONTEXT_3:
                if(locked == 0)
                {
                    if(context_position == 1)
                    {
                        knob(&wave_param[channel].mod_freq, CONT_LINE_NUM(key_freeze), channel);
                        if(KEY_ONE_OF_NUMERIC_KEYS)
                            wave_param[channel].mod_freq = enter_float(wave_param[channel].mod_freq, CONT_LINE_NUM(key_freeze));
                        lcd_param_and_val("mod freq=%f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].mod_freq);
                    }
                    else
                    {
                        wave_param[channel].mod_wave_num = enter_uint(wave_param[channel].mod_wave_num, CONT_LINE_NUM(key_freeze));
                        if(channel == 0)
                            lcd_param_and_val("mod num= %d", 'd', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].mod_wave_num);
                    }
                    refresh_configuration_all(p_fpga_params);
                }
                locked = 1;
                break;
                
            case KEY_CONTEXT_4:
                if(locked == 0)
                {
                    if(context_position == 1)
                    {
                        knob(&wave_param[channel].mod_dev, CONT_LINE_NUM(key_freeze), channel);
                        if(KEY_ONE_OF_NUMERIC_KEYS)
                            wave_param[channel].mod_dev = enter_float(wave_param[channel].mod_dev, CONT_LINE_NUM(key_freeze));
                        lcd_param_and_val("mod dev= %f", 'f', CONT_LINE_NUM(key_freeze), (void *)&wave_param[channel].mod_dev);
                    }
                    else
                        lcd_clear_line(4);                   
                    refresh_configuration_all(p_fpga_params);
                }
                locked = 1;
                break;
            
                                
            case KEY_CONTEXT_UP:
                if(locked == 0)
                {
                    if(context_position > 1)
                    {
                        --context_position;
                        
                        lcd_clear();
                        if(wave_param[channel].mod_type == MOD_NONE)
                            lcd_send_string("modul = NONE ", 13, 1, 0);
                        else if(wave_param[channel].mod_type == MOD_AM)
                            lcd_send_string("modul = AM   ", 13, 1, 0);
                        else if(wave_param[channel].mod_type == MOD_AM_SC)
                            lcd_send_string("modul = AM-SC", 13, 1, 0);
                        else if(wave_param[channel].mod_type == MOD_FM)
                            lcd_send_string("modul = FM   ", 13, 1, 0);
                        else if(wave_param[channel].mod_type == MOD_PM)
                            lcd_send_string("modul = PM   ", 13, 1, 0);
                        else if(wave_param[channel].mod_type == MOD_PWM)
                            lcd_send_string("modul = PWM  ", 13, 1, 0);
                        lcd_param_and_val("mod amp= %f", 'f', 2, (void *)&wave_param[channel].mod_ampl);
                        lcd_param_and_val("mod freq=%f", 'f', 3, (void *)&wave_param[channel].mod_freq);
                        lcd_param_and_val("mod dev= %f", 'f', 4, (void *)&wave_param[channel].mod_dev);
                    }
                }
                locked = 1;
                break;
            
            case KEY_CONTEXT_DOWN:
                if(locked == 0)
                {
                    if (context_position < context_screens)
                    {
                        ++context_position;
                        lcd_clear();
                        lcd_send_string("write modul. wave", 17, 1, 0);
                        if(wave_param[channel].mod_ext_not_int == 1)
                            lcd_send_string("mod ext", 7, 2, 0);
                        else
                            lcd_send_string("mod int", 7, 2, 0);
                        if(channel == 0)
                            lcd_param_and_val("mod num= %d", 'd', 3, (void *)&wave_param[channel].mod_wave_num);
                    }
                }
                locked = 1;
                break;
                
            case KEY_MOD_ON:
                if(locked == 0)
                {
                    if(!wave_param[channel].mod_type)
                    {
                        wave_param[channel].mod_type = last_mod_type;
                        led_matrix[LED_MOD_ON] = 1;
                    }
                    else
                    {
                        last_mod_type = wave_param[channel].mod_type;
                        wave_param[channel].mod_type = MOD_NONE;
                        led_matrix[LED_MOD_ON] = 0;
                    }
                    
                    if(wave_param[channel].mod_type == MOD_NONE)
                        lcd_send_string("modul = NONE ", 13, 1, 0);
                    else if(wave_param[channel].mod_type == MOD_AM)
                        lcd_send_string("modul = AM   ", 13, 1, 0);
                    else if(wave_param[channel].mod_type == MOD_AM_SC)
                        lcd_send_string("modul = AM-SC", 13, 1, 0);
                    else if(wave_param[channel].mod_type == MOD_FM)
                        lcd_send_string("modul = FM   ", 13, 1, 0);
                    else if(wave_param[channel].mod_type == MOD_PM)
                        lcd_send_string("modul = PM   ", 13, 1, 0);
                    else if(wave_param[channel].mod_type == MOD_PWM)
                        lcd_send_string("modul = PWM  ", 13, 1, 0);
                    
                    refresh_wave(channel, p_fpga_params);
                    send_configuration_fpga(*p_fpga_params, channel);
                }
                locked = 1;
                break;
                
            case KEY_EXIT:
                if(locked == 0)
                {
                    locked = 1;
                    return;
                }
                break;
            case -1:
                locked = 0;
                break;
        }
    }
}


void knob(double *processed_number, int disp_line, int channel)
{
    int rot_code;
    int last_rot_code;
    int rot_right = 0;
    int rot_left = 0;
    int deb_cnt;
    double incr_decr_val = 1;
    int i_lcd;
    char lcd_buf[40];
    int locked = 1;
    int key_freeze;
    double initial_processed_number;
    
    initial_processed_number = *processed_number;   //to allow clearing effects of rotating encoder
    while(g_key_numb != -1)
        ;
    
    last_rot_code = rot_code = (ROT_A_PORT | (ROT_B_PORT << 1));
    while(((key_freeze = g_key_numb) == -1) || (key_freeze == KEY_LEFT) || (key_freeze == KEY_RIGHT))
    {
        
        switch(g_key_numb)  //cases to execute if tact-switch pressed
        {
            case KEY_RIGHT:
                if(locked == 0)
                    incr_decr_val = incr_decr_val / 10;
                locked = 1;
                break;
            case KEY_LEFT:
                if(locked == 0)
                    incr_decr_val = incr_decr_val * 10;
                locked = 1;
                break;
            case -1:
                locked = 0;
                break;
        }
        
        
        if(deb_cnt < 1000)
            ++deb_cnt;
        if ((last_rot_code != (rot_code = (ROT_A_PORT | (ROT_B_PORT << 1)))) && (deb_cnt > 10))
        {
            if (((rot_code == 0x3) && (last_rot_code == 0x1))
            || ((rot_code == 0x1) && (last_rot_code == 0x0))
            || ((rot_code == 0x0) && (last_rot_code == 0x2))
            || ((rot_code == 0x2) && (last_rot_code == 0x3)))
            {
                ++rot_right;
                if (rot_right == 3)
                {
                    *processed_number = *processed_number + incr_decr_val;
                    sprintf(lcd_buf, "%f", *processed_number);
                    for (i_lcd = 0; (lcd_buf[i_lcd] != '\0') && ((NUM_START_COL + i_lcd) < 20); ++i_lcd)
                        lcd_send_char(lcd_buf[i_lcd], disp_line, NUM_START_COL + i_lcd);
                    rot_right = 0;
                    rot_left = 0;
                    refresh_wave(channel, &fpga_params);
                    send_configuration_fpga(fpga_params, channel);
                }
            }
            else if (((rot_code == 0x3) && (last_rot_code == 0x2))
                    || ((rot_code == 0x2) && (last_rot_code == 0x0))
                    || ((rot_code == 0x0) && (last_rot_code == 0x1))
                    || ((rot_code == 0x1) && (last_rot_code == 0x3)))
            {
                ++rot_left;
                if (rot_left == 3)
                {
                    *processed_number = *processed_number - incr_decr_val;
                    sprintf(lcd_buf, "%f", *processed_number);
                    for (i_lcd = 0; (lcd_buf[i_lcd] != '\0') && ((NUM_START_COL + i_lcd) < 20); ++i_lcd)
                        lcd_send_char(lcd_buf[i_lcd], disp_line, NUM_START_COL + i_lcd);
                    rot_right = 0;
                    rot_left = 0;
                    refresh_wave(channel, &fpga_params);
                    send_configuration_fpga(fpga_params, channel);
                }
            }
            deb_cnt = 0;
        }
        last_rot_code = rot_code;
    }
    if(key_freeze == KEY_CLEAR)
        *processed_number = initial_processed_number;
}

double enter_float(double initial_number, int disp_line)
{
    char digits_list[10] = {'0', '0', '0', '0', '0', '0', '0', '0', '0', '0'};
    char digit_keys[50];
    int i_digit = 0;
    int i_lcd;
    int locked = 0;
    int lock_point = 0;
    int point_pos = -1;
    double number_to_enter = 0;
    
    digit_keys[KEY_0] = '0';
    digit_keys[KEY_1] = '1';
    digit_keys[KEY_2] = '2';
    digit_keys[KEY_3] = '3';
    digit_keys[KEY_4] = '4';
    digit_keys[KEY_5] = '5';
    digit_keys[KEY_6] = '6';
    digit_keys[KEY_7] = '7';
    digit_keys[KEY_8] = '8';
    digit_keys[KEY_9] = '9';

    lcd_send_pos(disp_line, NUM_START_COL); //set position of the first character
    for(i_lcd = 0; i_lcd < (LCD_LENGTH - NUM_START_COL); ++i_lcd) //clear rest of characters
        lcd_send_next_char(' ');
    lcd_send_pos(disp_line, NUM_START_COL);   //set position

    
    while((g_key_numb != KEY_RETURN) && (g_key_numb != KEY_EXIT) && (i_digit < 10))
    {
        if(locked == 0)
        {
            if(     (g_key_numb == KEY_0) || (g_key_numb == KEY_1)
                ||  (g_key_numb == KEY_2) || (g_key_numb == KEY_3)
                ||  (g_key_numb == KEY_4) || (g_key_numb == KEY_5)
                ||  (g_key_numb == KEY_6) || (g_key_numb == KEY_7)
                ||  (g_key_numb == KEY_8) || (g_key_numb == KEY_9))
            {
                digits_list[i_digit] = digit_keys[g_key_numb];           
                lcd_send_char(digit_keys[g_key_numb], disp_line, NUM_START_COL + i_digit);
                ++i_digit;
                locked = 1;
            }
            else if((g_key_numb == KEY_POINT) && !lock_point)
            {
                digits_list[i_digit] = '.';
                point_pos = i_digit;
                lcd_send_char('.', disp_line, NUM_START_COL + i_digit);
                ++i_digit;
                lock_point = 1;
                locked = 1;
            }
            else if((g_key_numb == KEY_MINUS) && (i_digit == 0))
            {
                digits_list[i_digit] = '-';
                lcd_send_char('-', disp_line, NUM_START_COL + i_digit);
                ++i_digit;
                locked = 1;
            }
            else if((g_key_numb == KEY_BACKSPACE) && (i_digit > 0))
            {
                --i_digit;
                if(digits_list[i_digit] == '.')
                    lock_point = 0;
                digits_list[i_digit] = '0';
                lcd_send_char(' ', disp_line, NUM_START_COL + i_digit);
                locked = 1;
            }
        }
        if(g_key_numb == -1)
            locked = 0; // unlock when key released
    }
    
    if(!lock_point)
        digits_list[point_pos = i_digit] = '.';
    
    if(g_key_numb == KEY_RETURN)    //warning: this will work reliably unless the loop above takes more time than timer interrupt tick
    {
        for(i_digit = point_pos + 1; i_digit < 10; ++i_digit)
            number_to_enter += (double)(digits_list[i_digit] - '0') * pow(10, -(i_digit - point_pos));
        for(i_digit = point_pos - 1; (i_digit >= 0) && (digits_list[i_digit] != '-'); --i_digit)
            number_to_enter += (double)(digits_list[i_digit] - '0') * pow(10,  point_pos - i_digit - 1);
        if(digits_list[0] == '-')
            number_to_enter = - number_to_enter;
        return number_to_enter;
    }
    else
        return initial_number;  //if pressed esc - return initial number - no change
}

int enter_uint(int initial_number, int disp_line)
{
    char digits_list[10] = {'0', '0', '0', '0', '0', '0', '0', '0', '0', '0'};
    char digit_keys[50];
    int i_digit = 0;
    int i_lcd;
    int num_length;
    int locked = 0;
    int number_to_enter = 0;
    
    digit_keys[KEY_0] = '0';
    digit_keys[KEY_1] = '1';
    digit_keys[KEY_2] = '2';
    digit_keys[KEY_3] = '3';
    digit_keys[KEY_4] = '4';
    digit_keys[KEY_5] = '5';
    digit_keys[KEY_6] = '6';
    digit_keys[KEY_7] = '7';
    digit_keys[KEY_8] = '8';
    digit_keys[KEY_9] = '9';

    lcd_send_pos(disp_line, NUM_START_COL); //set position of the first character
    for(i_lcd = 0; i_lcd < (LCD_LENGTH - NUM_START_COL); ++i_lcd) //clear rest of characters
        lcd_send_next_char(' ');
    lcd_send_pos(disp_line, NUM_START_COL);   //set position

    while((g_key_numb != KEY_RETURN) && (g_key_numb != KEY_EXIT) && (i_digit < 10))
    {
        if(locked == 0)
        {
            if(     (g_key_numb == KEY_0) || (g_key_numb == KEY_1)
                ||  (g_key_numb == KEY_2) || (g_key_numb == KEY_3)
                ||  (g_key_numb == KEY_4) || (g_key_numb == KEY_5)
                ||  (g_key_numb == KEY_6) || (g_key_numb == KEY_7)
                ||  (g_key_numb == KEY_8) || (g_key_numb == KEY_9))
            {
                digits_list[i_digit] = digit_keys[g_key_numb];
                lcd_send_char(digit_keys[g_key_numb], disp_line, NUM_START_COL + i_digit);
                ++i_digit;
                locked = 1;
            }
            else if((g_key_numb == KEY_BACKSPACE) && (i_digit > 0))
            {
                --i_digit;
                digits_list[i_digit] = '0';
                lcd_send_char(' ', disp_line, NUM_START_COL + i_digit);
                locked = 1;
            }
        }
        if(g_key_numb == -1)
            locked = 0;
    }
    
    if(g_key_numb == KEY_RETURN)    //warning: this will work reliably unless the loop above takes more time than timer interrupt tick
    {
        --i_digit;
        for(num_length = i_digit; i_digit >= 0; --i_digit)
            number_to_enter += (int)(digits_list[i_digit] - '0') * pow(10, num_length - i_digit);
        return number_to_enter;
    }
    else
        return initial_number;  //if pressed esc - return initial number - no change
}


void led_funct_set(int funct)
{
    led_matrix[LED_FUNCT_SIN] = 0;
    led_matrix[LED_FUNCT_RECT] = 0;
    led_matrix[LED_FUNCT_TRI] = 0;
    led_matrix[LED_FUNCT_NOISE] = 0;
    led_matrix[LED_FUNCT_ARB] = 0;
    
    if(funct == FUNCT_SIN)
        led_matrix[LED_FUNCT_SIN] = 1;
    if(funct == FUNCT_RECT)
        led_matrix[LED_FUNCT_RECT] = 1;
    if(funct == FUNCT_TRI)
        led_matrix[LED_FUNCT_TRI] = 1;
    if(funct == FUNCT_NOISE)
        led_matrix[LED_FUNCT_NOISE] = 1;
    if(funct == FUNCT_ARB)
        led_matrix[LED_FUNCT_ARB] = 1;
}
