----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    00:10:53 07/19/2016 
-- Design Name: 
-- Module Name:    amplitude_offset - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity amplitude_offset is
    Port ( clk : in  STD_LOGIC;
           wr_en : in  STD_LOGIC;				
           wr : in  STD_LOGIC;				
           amplitude : in  STD_LOGIC_VECTOR (15 downto 0);
           offset : in  STD_LOGIC_VECTOR (15 downto 0);
           wave_in : in  STD_LOGIC_VECTOR (15 downto 0);
           wave_out : out  STD_LOGIC_VECTOR (13 downto 0);
			  am_mod_in : in  STD_LOGIC_VECTOR (15 downto 0);
			  am_set : in  STD_LOGIC_VECTOR (1 downto 0));
end amplitude_offset;

architecture Behavioral of amplitude_offset is

	signal amplitude_sig : std_logic_vector(17 DOWNTO 0) := (others => '0');
	signal offset_sig : std_logic_vector(15 downto 0) := (others => '0');
	signal scaled : std_logic_vector(35 downto 0) := (others => '0');
	signal scaled_and_mod : std_logic_vector(35 downto 0) := (others => '0');
	signal wave_out_sig : std_logic_vector(15 downto 0) := (others => '0');
	signal mod_mult : std_logic_vector(17 DOWNTO 0) := (others => '0');
	signal am_set_sig : std_logic_vector(1 DOWNTO 0) := (others => '0');
	signal am_mod_in_sig : std_logic_vector(17 DOWNTO 0) := (others => '0');
	
	COMPONENT multiplier
	PORT(
		clk : IN std_logic;
		a : IN std_logic_vector(17 downto 0);
		b : IN std_logic_vector(17 downto 0);          
		p : OUT std_logic_vector(35 downto 0)
		);
	END COMPONENT;


begin

	------------------------------------------------------------------------
	-- latch values received from microcontroller
	------------------------------------------------------------------------
	latch: process(wr) is
	begin
		if rising_edge(wr) then	--warning - this is not synchronous with fpga logic
			if wr_en = '1' then
				amplitude_sig(15 DOWNTO 0) <= amplitude(15 DOWNTO 0);
				offset_sig(15 DOWNTO 0) <= offset(15 DOWNTO 0);
				am_set_sig <= am_set;
			end if;
		end if;
	end process latch;
	------------------------------------------------------------------------


	------------------------------------------------------------------------
	-- scale, add offset and modulate amplitude of signal
	------------------------------------------------------------------------
	Inst_multiplier: multiplier PORT MAP(	
		clk => clk,
		a(17 downto 2) => wave_in,
		a(1 downto 0) => "00",
		b => amplitude_sig,
		p => scaled
	);

	am_mod_in_sig <= am_mod_in(15) & am_mod_in(15) & am_mod_in;	--extend length, don't loose sign
	--select modulation: "00" for no modulation, "01" for am, others for am-sc
	mod_mult <= "001000000000000000" when am_set_sig = "00" else	--no am
					std_logic_vector(signed(am_mod_in_sig) + 32768) when am_set_sig = "01" else	--am
					am_mod_in_sig;	--am-sc

	--multiply scaled signal by modulating signal to do AM/AM-SC modulation
	Inst_multiplier_AM_mod: multiplier PORT MAP(
		clk => clk,
		a(17 downto 2) => scaled(33 downto 18),
		a(1 downto 0) => "00",
		b => mod_mult,
		p => scaled_and_mod
	);

	--add offset to scaled (and maybe AM/AM-SC modulated) signal
	wave_out_sig <= std_logic_vector (signed(scaled_and_mod(32 downto 17)) + signed(offset_sig));
	--add additional offset that makes positive values for ADC from twos complement
	wave_out <= std_logic_vector (signed(wave_out_sig(15 downto 2)) + 8192);
	------------------------------------------------------------------------


end Behavioral;
