----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    12:23:28 03/12/2017 
-- Design Name: 
-- Module Name:    mod_interf - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity mod_interf is
    Port ( modmem_spi_in : in  STD_LOGIC;
           modmem_spi_out : out  STD_LOGIC;
           modmem_spi_cs : out  STD_LOGIC;
           modmem_spi_clk : out  STD_LOGIC;
           modmem_write_spi_in : in  STD_LOGIC;
           modmem_write_spi_cs : in  STD_LOGIC;
           modmem_write_spi_clk : in  STD_LOGIC;
           modadc_spi_in : in  STD_LOGIC;
           modadc_spi_cs : out  STD_LOGIC;
           modadc_spi_clk : out  STD_LOGIC;
           clk : in  STD_LOGIC;
           wr_en : in  STD_LOGIC;
           wr : in  STD_LOGIC;
			  mod_source_sel : in  STD_LOGIC;
           modmem_rw : in  STD_LOGIC;
           mod_wave : out  STD_LOGIC_VECTOR (31 downto 0);
			  mod_ampl : in STD_LOGIC_VECTOR (15 downto 0);
			  mod_ampl_pow : in STD_LOGIC_VECTOR (3 downto 0);
			  mod_wave_num : in STD_LOGIC_VECTOR (7 downto 0);
			  mod_freq_set : in  STD_LOGIC_VECTOR (31 downto 0);
           wr_en_mod_freq : in  STD_LOGIC;
			  wr_en_adc_cal : in  STD_LOGIC;
			  adc_offs_cal : in  STD_LOGIC_VECTOR (15 downto 0);
			  adc_gain_cal : in  STD_LOGIC_VECTOR (15 downto 0);
			  ov : out STD_LOGIC;
			  trig_out : out STD_LOGIC);
end mod_interf;

architecture Behavioral of mod_interf is

	signal modmem_rw_sig : std_logic := '0';
	signal modmem_read_spi_out : std_logic := '0';
	signal modmem_read_spi_in : std_logic := '0';
	signal modmem_read_spi_cs : std_logic := '1';
	signal modmem_read_spi_clk : std_logic := '0';
	signal mem_spi_data_write : std_logic_vector(7 downto 0) := (others => '0');
	signal mem_spi_reg_write : std_logic_vector(7 downto 0) := (others => '0');
	signal mem_spi_reg_read : std_logic_vector(7 downto 0) := (others => '0');
	signal adc_spi_reg_read : std_logic_vector(15 downto 0) := (others => '0');
	signal bit_cnt : unsigned(7 downto 0) := (others => '0');
	signal modmem_addr : std_logic_vector(23 downto 0) := (others => '0');
	signal tmp : std_logic_vector(7 downto 0) := (others => '0');
	signal mod_flash_wave_sig : std_logic_vector(15 downto 0) := (others => '0');
	signal mod_adc_wave_sig : std_logic_vector(15 downto 0) := (others => '0');
	signal mod_adc_raw_sig : std_logic_vector(15 downto 0) := (others => '0');
	signal mod_wave_sig : std_logic_vector(15 downto 0) := (others => '0');
	signal mod_ampl_sig : std_logic_vector(15 downto 0) := (others => '0');
	signal mod_ampl_pow_sig : std_logic_vector(3 downto 0) := (others => '0');
	signal mod_phase_acc : std_logic_vector(32 downto 0) := (others => '0');
	signal mod_freq_sig : std_logic_vector(31 downto 0) := (others => '0');
	signal mod_pa_msbits : std_logic_vector(12 downto 0) := (others => '0');
	signal mod_wave_num_sig : std_logic_vector(7 downto 0) := (others => '0');
	signal mod_wave_scaled_sig : std_logic_vector(15 downto 0) := (others => '0');
	signal mod_wave_scaled_signed_sig : signed(31 downto 0) := (others => '0');
	signal mod_source_sel_sig : std_logic := '0';
	signal asdf1 : std_logic_vector(0 downto 0);
	signal asdf2 : std_logic_vector(18 downto 0);
	signal asdf3 : std_logic_vector(35 downto 30);
	signal asdf4 : std_logic_vector(13 downto 0);
	signal cal_adc_offs_sig : std_logic_vector(15 downto 0) := (others => '0');
	signal cal_adc_gain_sig : std_logic_vector(15 downto 0) := (others => '0');
	signal mod_adc_offcal_sig : std_logic_vector(17 downto 0) := (others => '0');
	signal mod_adc_unlim_sig : std_logic_vector(21 downto 0) := (others => '0');
	
	COMPONENT multiplier
	PORT(
		clk : IN std_logic;
		a : IN std_logic_vector(17 downto 0);
		b : IN std_logic_vector(17 downto 0);          
		p : OUT std_logic_vector(35 downto 0)
		);
	END COMPONENT;
	
begin

	------------------------------------------------------------------------
	--latch values received from microcontroller
	------------------------------------------------------------------------
	latch: process(wr) is
	begin
		if rising_edge(wr) then	--warning - this is not synchronous with fpga logic
			if wr_en = '1' then
				modmem_rw_sig <= modmem_rw;
				mod_ampl_sig <= mod_ampl;
				mod_ampl_pow_sig <= mod_ampl_pow;
				mod_source_sel_sig <= mod_source_sel;
				mod_wave_num_sig <= mod_wave_num(7 downto 0);
			elsif wr_en_mod_freq = '1' then
				mod_freq_sig <= mod_freq_set;
			elsif wr_en_adc_cal = '1' then
				cal_adc_offs_sig <= adc_offs_cal;
				cal_adc_gain_sig <= adc_gain_cal;
			end if;
		end if;
	end process latch;
	------------------------------------------------------------------------


	------------------------------------------------------------------------
	-- read / write memory with modulating wave
	------------------------------------------------------------------------	
	modmem_spi_out <= modmem_read_spi_out when modmem_rw_sig = '0' else
							modmem_write_spi_in;
   modmem_spi_cs 	<= modmem_read_spi_cs when modmem_rw_sig = '0' else
							modmem_write_spi_cs;
   modmem_spi_clk <= modmem_read_spi_clk when modmem_rw_sig = '0' else
							modmem_write_spi_clk;
	modmem_read_spi_clk <= clk;

	mem_process: process(clk) is
	begin
	if rising_edge(clk) then
		bit_cnt <= bit_cnt + 1;
		if bit_cnt = 0 then
			modmem_read_spi_cs <= '0';
			mem_spi_data_write <= x"03";
		elsif bit_cnt = 8 then
			mem_spi_data_write <= modmem_addr(23 downto 16);
		elsif bit_cnt = 16 then
			mem_spi_data_write <= modmem_addr(15 downto 8);
		elsif bit_cnt = 24 then
			mem_spi_data_write <= modmem_addr(7 downto 0);
		elsif bit_cnt = 41 then
			tmp <= mem_spi_reg_read;
		elsif bit_cnt = 49 then
			mod_flash_wave_sig(7 downto 0) <= tmp;
			mod_flash_wave_sig(15 downto 8) <= mem_spi_reg_read;
		elsif bit_cnt = 57 then
			modmem_read_spi_cs <= '1';
		elsif bit_cnt = 64 then
			bit_cnt <= (others => '0');
			modmem_addr(20 downto 1) <= std_logic_vector(unsigned("000000000" & mod_pa_msbits) + unsigned(mod_wave_num_sig & "0000000000000"));	--(20 downto 1) to take every second address, because two bytes are used for one sample (14 bits)
		end if;
	end if;
	end process mem_process;
	
	byte_write: process(clk) is
	begin
	if falling_edge(clk) then
		if bit_cnt = 1 OR bit_cnt = 9 OR bit_cnt = 17 OR bit_cnt = 25 then
			mem_spi_reg_write <= mem_spi_data_write;
		else
			mem_spi_reg_write(7 downto 1) <= mem_spi_reg_write(6 downto 0);
			mem_spi_reg_write(0) <= '0';
		end if;
	end if;
	end process byte_write;
	modmem_read_spi_out <= mem_spi_reg_write(7);
	
	byte_read: process(clk) is
	begin
	if rising_edge(clk) then
		mem_spi_reg_read(0) <= modmem_spi_in;
		mem_spi_reg_read(7 downto 1) <= mem_spi_reg_read(6 downto 0);
	end if;
	end process byte_read;
	------------------------------------------------------------------------


	------------------------------------------------------------------------
	-- read modulating signal from adc
	------------------------------------------------------------------------
	adc_read: process(clk) is
		variable bit_cnt_adc : unsigned(7 downto 0) := (others => '0');
	begin
	if rising_edge(clk) then
		adc_spi_reg_read(0) <= modadc_spi_in;
		adc_spi_reg_read(15 downto 1) <= adc_spi_reg_read(14 downto 0);
		if bit_cnt_adc = 1 then
			mod_adc_raw_sig <= adc_spi_reg_read;
		elsif bit_cnt_adc = 16 then
			modadc_spi_cs <= '1';			
		elsif bit_cnt_adc = 20 then
			bit_cnt_adc := (others => '0');
			modadc_spi_cs <= '0';
		end if;
		bit_cnt_adc := bit_cnt_adc + 1;
	end if;
	end process adc_read;
	modadc_spi_clk <= clk;
	------------------------------------------------------------------------
	

	------------------------------------------------------------------------
	-- process adc signal
	------------------------------------------------------------------------
	mod_adc_offcal_sig <= std_logic_vector(signed("00" & mod_adc_raw_sig) + signed(cal_adc_offs_sig(15) & cal_adc_offs_sig(15) & cal_adc_offs_sig) - 32768);
	Inst_multiplier_cal: multiplier PORT MAP(
		clk => clk,
		a(17 downto 0) => mod_adc_offcal_sig,
		b(15 downto 0) => cal_adc_gain_sig,
		b(17 downto 16) => "00",
		p(35 downto 14) => mod_adc_unlim_sig(21 downto 0),
		p(13 downto 0) => asdf4
	);
	------------------------------------------------------------------------

	------------------------------------------------------------------------
	--overvoltage limiter and indicator
	------------------------------------------------------------------------
	overvoltage: process (clk) is
		variable ovcnt : unsigned(23 downto 0) := (others => '0');
	begin
		if rising_edge (clk) then
			if signed(mod_adc_unlim_sig(21 downto 0)) > 32767 then
				mod_adc_wave_sig <= "0111111111111111";
				ov <= '1';
				ovcnt := (others => '1');
			elsif signed(mod_adc_unlim_sig(21 downto 0)) < -32768 then
				mod_adc_wave_sig <= "1000000000000000";
				ov <= '1';
				ovcnt := (others => '1');
			else
				mod_adc_wave_sig <= mod_adc_unlim_sig(15 downto 0);
				if ovcnt > 0 then
					ovcnt := ovcnt - 1;
				else
					ov <= '0';
				end if;
			end if;
		end if;
	end process overvoltage;
	------------------------------------------------------------------------	
	
	
	------------------------------------------------------------------------
	-- select modulating signal and adjust level
	------------------------------------------------------------------------
	mod_wave_sig <= mod_flash_wave_sig when mod_source_sel_sig = '0' else
						 mod_adc_wave_sig;
	Inst_multiplier_modulation: multiplier PORT MAP(
		clk => clk,
		a(17 downto 2) => mod_ampl_sig,
		a(1 downto 0) => "00",
		b(17 downto 2) => mod_wave_sig,
		b(1 downto 0) => "00",
		p(34 downto 19) => mod_wave_scaled_sig(15 downto 0) ,
		p(35 downto 35) => asdf1,
		p(18 downto 0) => asdf2
	);
	mod_wave_scaled_signed_sig <= resize(signed(mod_wave_scaled_sig), mod_wave_scaled_signed_sig'length);
	mod_wave <= std_logic_vector(shift_left(mod_wave_scaled_signed_sig, to_integer(unsigned(mod_ampl_pow_sig))));
	------------------------------------------------------------------------

	------------------------------------------------------------------------
	--modulation waveform phase accumullator
	------------------------------------------------------------------------
	mod_phase_cnt: process (clk) is
	begin
		if rising_edge (clk) then
			mod_phase_acc <= std_logic_vector(unsigned(mod_phase_acc) + unsigned(mod_freq_sig));
		end if;
	end process mod_phase_cnt;
	mod_pa_msbits <= mod_phase_acc(32 downto 20);
	trig_out <= mod_pa_msbits(mod_pa_msbits'length - 1);	-- trigger out
	------------------------------------------------------------------------	
	
	
	
end Behavioral;

