----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    22:57:15 01/29/2017 
-- Design Name: 
-- Module Name:    mux_2x13b - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity mux_2x13b is
    Port ( in1 : in  STD_LOGIC_VECTOR (12 downto 0);
           in2 : in  STD_LOGIC_VECTOR (12 downto 0);
			  in1clk : in  STD_LOGIC;
           in2clk : in  STD_LOGIC;
           sel : in  STD_LOGIC;
			  wr_en : in  STD_LOGIC;
           wr : in  STD_LOGIC;
           m_out : out  STD_LOGIC_VECTOR (12 downto 0);
			  m_outclk : out  STD_LOGIC;
			  trig_out : out  STD_LOGIC;
			  phase : in  STD_LOGIC_VECTOR (12 downto 0);
			  phase_mod_in : in  STD_LOGIC_VECTOR (12 downto 0);
			  phase_mod_on : in  STD_LOGIC);
end mux_2x13b;

architecture Behavioral of mux_2x13b is

	signal sel_sig : std_logic := '0';
	signal m_outclk_sig : std_logic := '0';
	signal m_out_sig : std_logic_vector(12 downto 0) := (others => '0');
	signal phase_sig : std_logic_vector(12 downto 0) := (others => '0');
	signal phase_mod_on_sig : std_logic := '0';
begin


	------------------------------------------------------------------------
	--latch values received from microcontroller
	------------------------------------------------------------------------
	latch: process(wr) is
	begin
		if rising_edge(wr) then
			if wr_en = '1' then
				sel_sig <= sel;
				phase_sig <= phase;
				phase_mod_on_sig <= phase_mod_on;
			end if;
		end if;
	end process latch;
	------------------------------------------------------------------------
	
	
	------------------------------------------------------------------------
	-- clock multiplexer
	------------------------------------------------------------------------
	BUFGMUX_inst : BUFGMUX
	port map (
		O => m_outclk_sig,	-- Clock MUX output
		I0 => in1clk,	-- Clock0 input
		I1 => in2clk,	-- Clock1 input
		S => sel_sig	-- Clock select input
	);
	------------------------------------------------------------------------


	------------------------------------------------------------------------
	-- address multiplexer and phase shifting
	------------------------------------------------------------------------
	sel_proc: process(m_outclk_sig) is
	begin
		if rising_edge(m_outclk_sig) then
			if sel_sig = '0' then
				if phase_mod_on_sig = '1' then
					m_out_sig <= std_logic_vector(unsigned(in1) + unsigned(phase_sig) + unsigned(phase_mod_in));
				else
					m_out_sig <= std_logic_vector(unsigned(in1) + unsigned(phase_sig));
				end if;
				trig_out <= in1(in2'length - 1);
			else
				if phase_mod_on_sig = '1' then
					m_out_sig <= std_logic_vector(unsigned(in2) + unsigned(phase_sig) + unsigned(phase_mod_in));
				else
					m_out_sig <= std_logic_vector(unsigned(in2) + unsigned(phase_sig));
				end if;
				trig_out <= in2(in2'length - 1);
			end if;
		end if;
	end process sel_proc;
	
	m_out <= m_out_sig;
	m_outclk <= m_outclk_sig;
	------------------------------------------------------------------------



end Behavioral;

