----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    19:40:15 07/19/2016 
-- Design Name: 
-- Module Name:    wave_clk - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity wave_clk is
    Port ( clk_tcxo : in  STD_LOGIC;
           freq_set : in  STD_LOGIC_VECTOR (31 downto 0);
           wr_en : in  STD_LOGIC;
           wr : in  STD_LOGIC;
           pll_ext1 : in  STD_LOGIC;
           pll_ext2 : in  STD_LOGIC;
           from_pll_ext1 : in  STD_LOGIC;
           from_pll_ext2 : in  STD_LOGIC;
			  pa_msbits_out : out STD_LOGIC_VECTOR (12 downto 0);
           wave_clk : out  STD_LOGIC;
           to_pll_ext1 : out  STD_LOGIC;
           to_pll_ext2 : out  STD_LOGIC;
			  clk_main_in : in  STD_LOGIC;
			  clk_main_out : out  STD_LOGIC;
			  freq_mod_on : in  STD_LOGIC;
			  freq_dev : in  STD_LOGIC_VECTOR (31 downto 0));
end wave_clk;

architecture Behavioral of wave_clk is

	signal clk_pa_out : std_logic := '0';
	signal phase_acc : std_logic_vector(32 downto 0) := (others => '0');
	signal freq_set_sig : std_logic_vector(31 downto 0) := (others => '0');
	signal pll_ext1_sig : std_logic := '0';
	signal pll_ext2_sig : std_logic := '0';
	signal wave_clk_buf_in : std_logic := '0';
	signal freq_mod_on_sig : std_logic := '0';
	signal phase_incr : std_logic_vector(31 downto 0) := (others => '0');
	
	COMPONENT pa_clk_dcm
	PORT(
		CLKIN_IN : IN std_logic;          
		CLKFX_OUT : OUT std_logic
		);
	END COMPONENT;
	
begin

	------------------------------------------------------------------------
	-- generate phase accumulator input frequency
	------------------------------------------------------------------------
	Inst_pa_clk_dcm: pa_clk_dcm PORT MAP(
		CLKIN_IN => clk_tcxo,
		CLKFX_OUT => clk_main_out
	);
	------------------------------------------------------------------------
	
	------------------------------------------------------------------------
	-- latch values received from microcontroller
	------------------------------------------------------------------------
	latch: process(wr) is
	begin
		if rising_edge(wr) then
			if wr_en = '1' then
				freq_set_sig <= freq_set;
				pll_ext1_sig <= pll_ext1;
				pll_ext2_sig <= pll_ext2;
				freq_mod_on_sig <= freq_mod_on;
			end if;
		end if;
	end process latch;
	------------------------------------------------------------------------
	

	------------------------------------------------------------------------
	-- add modulating signal to phase increment
	------------------------------------------------------------------------
	pipeline_reg_freq_set: process (clk_main_in) is
	begin
		if rising_edge (clk_main_in) then
			if freq_mod_on_sig = '1' then
				phase_incr <= std_logic_vector(signed(freq_set_sig) + signed(freq_dev));
			else
				phase_incr <= freq_set_sig;
			end if;
		end if;
	end process pipeline_reg_freq_set;
	------------------------------------------------------------------------

	
	------------------------------------------------------------------------
	-- phase accumullator
	------------------------------------------------------------------------
	phase_cnt: process (clk_main_in) is
	begin
		if rising_edge (clk_main_in) then
			phase_acc <= std_logic_vector(signed(phase_acc) + signed(phase_incr));
		end if;
	end process phase_cnt;
	clk_pa_out <= phase_acc(32);
	pa_msbits_out <= phase_acc(32 downto 20);
	------------------------------------------------------------------------
	

	------------------------------------------------------------------------
	-- select pll
	------------------------------------------------------------------------
	to_pll_ext1 <= clk_pa_out;
	to_pll_ext2 <= clk_pa_out when (pll_ext2_sig = '1' AND pll_ext1_sig = '0') else
						from_pll_ext1 when (pll_ext2_sig = '1' AND pll_ext1_sig = '1') else
						'0';
	wave_clk_buf_in <=	from_pll_ext1 when (pll_ext1_sig = '1' AND pll_ext2_sig = '0') else 
								from_pll_ext2 when pll_ext2_sig = '1' else 
								clk_pa_out;
	------------------------------------------------------------------------

	
	------------------------------------------------------------------------
	-- clock buffer for generated clock
	------------------------------------------------------------------------
	BUFG_inst : BUFG
	port map (
		O => wave_clk,			-- Clock buffer output
		I => wave_clk_buf_in	-- Clock buffer input
	);
	------------------------------------------------------------------------	
	
	
end Behavioral;

