----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    23:57:19 07/21/2016 
-- Design Name: 
-- Module Name:    wave_mem_gen - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity wave_mem_gen is
    Port ( clk_wav_gen : in  STD_LOGIC;
           clk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           wr : in  STD_LOGIC;
           wav_gen_scale : in  STD_LOGIC_VECTOR (17 downto 0);
           wav_gen_offset : in  STD_LOGIC_VECTOR (4 downto 0);
			  sin_tri_select : in  STD_LOGIC;
           clk_lut_interf : in  STD_LOGIC;
           eeprom_c : in  STD_LOGIC;
           eeprom_q : in  STD_LOGIC;
           eeprom_d : out  STD_LOGIC;
           data_out : out  STD_LOGIC_VECTOR (15 downto 0);
           addr_out : out  STD_LOGIC_VECTOR (12 downto 0));
end wave_mem_gen;

architecture Behavioral of wave_mem_gen is
	signal wav_gen_scale_sig : std_logic_vector(17 downto 0) := (others => '0');
	signal wav_gen_offset_sig : std_logic_vector(4 downto 0) := (others => '0');
	signal data_triangle_scaled : std_logic_vector(30 downto 0) := (others => '0');
	signal data_triangle_out : std_logic_vector(15 downto 0) := (others => '0');
	signal data_sinus_out : std_logic_vector(15 downto 0) := (others => '0');
	signal memory_data_counter_sig : std_logic_vector(13 downto 0) := (others => '0');
	signal tri_rampdown_offs : signed(17 downto 0) := (others => '0');
begin


	latch: process(clk) is
	begin
		if rising_edge(clk) then
			if wr = '1' then
				wav_gen_scale_sig <= wav_gen_scale;
				wav_gen_offset_sig <= wav_gen_offset;
			end if;
		end if;
	end process latch;


	mem_addr_cnt: process(clk_wav_gen, reset) is
		variable memory_counter: unsigned(13 downto 0);
	begin
		if reset = '1' and wav_gen_offset_sig(4) = '0' then
			memory_counter := (others => '0');
		elsif rising_edge(clk_wav_gen) then
			memory_counter := memory_counter + 1;
		end if;
		addr_out <= std_logic_vector(memory_counter(12 downto 0));
	end process mem_addr_cnt;

	mem_data_cnt: process(clk_wav_gen, reset) is
		variable memory_counter: unsigned(13 downto 0);
	begin
		if reset = '1' then
			memory_counter := (others => '0');
		elsif rising_edge(clk_wav_gen) then
			memory_counter := memory_counter + 1;
		end if;
		memory_data_counter_sig <= std_logic_vector(memory_counter);
	end process mem_data_cnt;
	
	
	tri_rampdown_offs <= "100000000000000000" when wav_gen_offset(4) = '0' else 
								"011111111111111111";
	
	data_triangle_scaled <= std_logic_vector(tri_rampdown_offs + signed(memory_data_counter_sig) * signed(wav_gen_scale_sig));
	
	data_triangle_out <= data_triangle_scaled(17 downto 2) when wav_gen_offset(3 downto 0) = "0000" else
								data_triangle_scaled(18 downto 3) when wav_gen_offset(3 downto 0) = "0001" else
								data_triangle_scaled(19 downto 4) when wav_gen_offset(3 downto 0) = "0010" else
								data_triangle_scaled(20 downto 5) when wav_gen_offset(3 downto 0) = "0011" else
								data_triangle_scaled(21 downto 6) when wav_gen_offset(3 downto 0) = "0100" else
								data_triangle_scaled(22 downto 7) when wav_gen_offset(3 downto 0) = "0101" else
								data_triangle_scaled(23 downto 8) when wav_gen_offset(3 downto 0) = "0110" else
								data_triangle_scaled(24 downto 9) when wav_gen_offset(3 downto 0) = "0111" else
								data_triangle_scaled(25 downto 10) when wav_gen_offset(3 downto 0) = "1000" else
								data_triangle_scaled(26 downto 11) when wav_gen_offset(3 downto 0) = "1001" else
								data_triangle_scaled(27 downto 12) when wav_gen_offset(3 downto 0) = "1010" else
								data_triangle_scaled(28 downto 13) when wav_gen_offset(3 downto 0) = "1011" else
								data_triangle_scaled(29 downto 14) when wav_gen_offset(3 downto 0) = "1100" else
								data_triangle_scaled(30 downto 15) when wav_gen_offset(3 downto 0) = "1101" else
								(others => 'X');
	
	data_out <= data_triangle_out when sin_tri_select = '0' else 
					data_sinus_out;
	
	
	
end Behavioral;

