#include <stdio.h>   /* Standard input/output definitions */
#include <string.h>  /* String function definitions */
#include <unistd.h>  /* UNIX standard function definitions */
#include <fcntl.h>   /* File control definitions */
#include <errno.h>   /* Error number definitions */
#include <termios.h> /* POSIX terminal control definitions */
#include <stdint.h>


int main(void)
{
	int fd;	//port file descriptor
	int n;	//return value of some functions
    struct termios tty;	//port configuration
	char wave_filename[256];	//name of csv wave file to send
	char wave_num;	//location of wave in generators memory
	char port_path[256]	= "0";	//path of port file
	int32_t wavebuf[8192];	//buffer with wave read from file
	char buf256[256];	//buffer with piece of wave
	int i_copyfile;
	int i;
	int j;
	char preamble[6] = {'A', 'A', 'A', 'A', 'A', 'A'};	//string send as preamble to avoid unwanted start of transmission


	printf("enter number of wave in generators memory\n");
	scanf("%c", &wave_num);
	printf("enter csv file name with wave\n");
	scanf("%s", wave_filename);
	printf("enter port file (put 0 to default /dev/ttyACM0)\n");
	scanf("%s", port_path);
	printf("\n\n");
	if(port_path[0] == '0')
		strcpy(port_path, "/dev/ttyACM0");

	FILE *file = fopen(wave_filename, "r");	//open csv file with wave
    if (file == NULL)
    {
		printf("Error from fopen: %s\n", strerror(errno));
        return 1;
    }

	for(i_copyfile = 0; i_copyfile < 8192; ++i_copyfile)	//copy csv file to buffer
		fscanf(file, "%d", wavebuf + i_copyfile);      
    fclose(file);
	
	fd = open(port_path, O_RDWR | O_NOCTTY | O_NDELAY);
	if (fd == -1)
		printf("Error from open: %s\n", strerror(errno));
	else
    	fcntl(fd, F_SETFL, 0);

	tcgetattr(fd, &tty);
    cfsetospeed(&tty, B9600);
    cfsetispeed(&tty, B9600);
    tty.c_cflag |= (CLOCAL | CREAD);    /* ignore modem controls */
    tty.c_cflag &= ~CSIZE;
    tty.c_cflag |= CS8;         /* 8-bit characters */
    tty.c_cflag &= ~PARENB;     /* no parity bit */
    tty.c_cflag &= ~CSTOPB;     /* only need 1 stop bit */
    tty.c_cflag &= ~CRTSCTS;    /* no hardware flowcontrol */
    tty.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
	tty.c_oflag &= ~OPOST;

    if (tcsetattr(fd, TCSANOW, &tty) != 0) {
        printf("Error from tcsetattr: %s\n", strerror(errno));
        return -1;
    }

	tcflush(fd, TCIOFLUSH);


    if (write(fd, preamble, strlen(preamble)) == -1)	//send preamble
	{
        printf("Error from write: %s\n", strerror(errno));
        return -1;
    }
	tcdrain(fd);
	usleep(2000000);	//if not wait long sth goes wrong

    if ((n=write(fd, &wave_num, 1)) == -1)	//send number of wave in generators memory (0-9)
	{
        printf("Error from write: %s\n", strerror(errno));
        return -1;
    }
	tcdrain(fd);
	usleep(2000000);	//if not wait long sth goes wrong


	for(i = 0; i < 64; ++i)	//send data in 256 bytes chunks
	{
		for(j = 0; j < 256; ++j)
			if(j % 2 == 0)
				buf256[j] = 0x00FF & wavebuf[j / 2 + 128 * i];
			else
				buf256[j] = (0xFF00 & wavebuf[j / 2 + 128 * i]) >> 8;

		if (write(fd, buf256, 256) == -1)	//write 256 bytes chunk to port
		{
		    printf("Error from write: %s\n", strerror(errno));
		    return -1;
		}
		tcdrain(fd);
		usleep(100000);
	}

	close(fd);
	return 0;
}

